import unittest

from action_hero.types_utils import (
    is_convertible_to_UUID,
    is_convertible_to_float,
    is_convertible_to_int,
    is_truthy,
)


class TestIsConvertibleToInt(unittest.TestCase):
    def test_on_valid_value_zero(self):
        value = "0"
        self.assertTrue(is_convertible_to_int(value))

    def test_on_valid_value_positive_integer(self):
        value = "3"
        self.assertTrue(is_convertible_to_int(value))

    def test_on_valid_value_negative_integer(self):
        value = "-20"
        self.assertTrue(is_convertible_to_int(value))

    def test_on_invalid_value_string(self):
        value = "banana"
        self.assertFalse(is_convertible_to_int(value))

    def test_on_invalid_value_float(self):
        value = "3.40"
        self.assertFalse(is_convertible_to_int(value))

    def test_on_invalid_value_blank(self):
        value = ""
        self.assertFalse(is_convertible_to_int(value))


class TestIsConvertibleToFloat(unittest.TestCase):
    def test_on_valid_value_zero(self):
        value = "0"
        self.assertTrue(is_convertible_to_float(value))

    def test_on_valid_value_positive_integer(self):
        value = "3"
        self.assertTrue(is_convertible_to_float(value))

    def test_on_valid_value_negative_integer(self):
        value = "-20"
        self.assertTrue(is_convertible_to_float(value))

    def test_on_valid_value_positive_float(self):
        value = "234.5"
        self.assertTrue(is_convertible_to_float(value))

    def test_on_valid_value_negative_float(self):
        value = "-20.123"
        self.assertTrue(is_convertible_to_float(value))

    def test_on_invalid_value_string(self):
        value = "banana"
        self.assertFalse(is_convertible_to_float(value))

    def test_on_invalid_value_blank(self):
        value = ""
        self.assertFalse(is_convertible_to_float(value))


class TestIsTruthy(unittest.TestCase):
    def test_on_truthy_value_strings(self):
        value = "true"
        self.assertTrue(is_truthy(value))

    def test_on_falsy_value_strings_blank(self):
        value = ""
        self.assertFalse(is_truthy(value))

    def test_on_falsy_value_floats_zeroes_list(self):
        values = ["0", "0.0"]
        [self.assertFalse(is_truthy(value)) for value in values]


class TestIsConvertibleToUUID(unittest.TestCase):
    def test_on_valid_UUID_value(self):
        value = "9cc79eb2-ed8a-4216-b1a8-9ab65bc4d92b"
        self.assertTrue(is_convertible_to_UUID(value))

    def test_on_valid_UUID_values_list(self):
        values = [
            "9cc79eb2-ed8a-4216-b1a8-9ab65bc4d92b",
            "16fd2706-8baf-433b-82eb-8c7fada847da",
            "527e620b-180d-49d0-9811-8c4e0aa7f095",
        ]
        [self.assertTrue(is_convertible_to_UUID(value)) for value in values]

    def test_on_invalid_UUID_value(self):
        value = "monday-tuesday-wednesday-thursday-friday-saturday-sunday"
        self.assertFalse(is_convertible_to_UUID(value))

    def test_on_invalid_UUID_values_list(self):
        values = ["chocolate-pudding", "231", "002345234"]
        self.assertIn(False, [is_convertible_to_UUID(v) for v in values])

    def test_on_mixed_valid_and_invalid_UUID_values_list(self):

        values = [
            "9cc79eb2-ed8a-4216-b1a8-9ab65bc4d92b",
            "chocolate-pudding",
            "16fd2706-8baf-433b-82eb-8c7fada847da",
            "231",
            "527e620b-180d-49d0-9811-8c4e0aa7f095",
            "002345234",
        ]
        self.assertIn(False, [is_convertible_to_UUID(v) for v in values])



import unittest
import tempfile
import shutil
import os

from action_hero.path_utils import (
    add_execute_permission,
    create_directory,
    create_file,
    get_extension,
    is_empty_file,
    is_executable_directory,
    is_executable_file,
    is_executable_path,
    is_existing_directory,
    is_existing_file,
    is_existing_or_creatable_path,
    is_existing_path,
    is_readable_directory,
    is_readable_file,
    is_readable_path,
    is_symbolic_link,
    is_valid_directory,
    is_valid_file,
    is_valid_path,
    is_writable_directory,
    is_writable_file,
    is_writable_path,
    remove_execute_permission,
    remove_read_permission,
    remove_write_permission,
    resolve_path,
)


class TestExistenceUtils(unittest.TestCase):
    def test_on_exisiting_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_existing_directory(dir1))

    def test_on_nonexisting_directory(self):
        dir1 = tempfile.mkdtemp()
        os.rmdir(dir1)

        self.assertFalse(is_existing_directory(dir1))

    def test_on_exisiting_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_existing_file(file1.name))

    def test_on_nonexisting_file(self):
        # Create and Remove a file to confirm it doesnt exist
        file1 = tempfile.mkstemp()[1]
        os.remove(file1)

        self.assertFalse(is_existing_file(file1))

    def test_on_exisiting_path_as_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_existing_path(file1.name))

    def test_on_nonexisting_path_as_file(self):
        # Create and Remove a file to confirm it doesnt exist
        file1 = tempfile.mkstemp()[1]
        os.remove(file1)

        self.assertFalse(is_existing_path(file1))

    def test_on_exisiting_path_as_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_existing_path(dir1))

    def test_on_nonexisting_path_as_directory(self):
        # Create and Remove a directory to confirm it doesnt exist
        dir1 = tempfile.mkdtemp()
        os.rmdir(dir1)

        self.assertFalse(is_existing_path(dir1))


class TestWritableUtils(unittest.TestCase):
    def test_on_writable_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_writable_directory(dir1))

    def test_on_unwritable_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            remove_write_permission(dir1)
            self.assertFalse(is_writable_directory(dir1))

    def test_on_writable_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_writable_file(file1.name))

    def test_on_unwritable_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            remove_write_permission(file1.name)
            self.assertFalse(is_writable_file(file1.name))

    def test_on_writable_pasth_as_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_writable_path(dir1))

    def test_on_unwritable_path_as_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            remove_write_permission(dir1)
            self.assertFalse(is_writable_path(dir1))

    def test_on_writable_path_as_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_writable_path(file1.name))

    def test_on_unwritable_path_as_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            remove_write_permission(file1.name)
            self.assertFalse(is_writable_path(file1.name))


class TestReadableUtils(unittest.TestCase):
    def test_on_readable_directory(self):
        # Specify temporary directory
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_readable_directory(dir1))

    def test_on_unreadable_directory(self):
        dir1 = tempfile.mkdtemp()
        remove_read_permission(dir1)
        self.assertFalse(is_readable_directory(dir1))
        os.rmdir(dir1)

    def test_on_readable_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_readable_file(file1.name))

    def test_on_unreadable_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            remove_read_permission(file1.name)
            self.assertFalse(is_readable_file(file1.name))

    def test_on_readable_path_as_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_readable_path(dir1))

    def test_on_unreadable_path_as_directory(self):
        dir1 = tempfile.mkdtemp()
        remove_read_permission(dir1)
        self.assertFalse(is_readable_directory(dir1))
        os.rmdir(dir1)

    def test_on_readable_path_as_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_readable_path(file1.name))

    def test_on_unreadable_path_as_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            remove_read_permission(file1.name)
            self.assertFalse(is_readable_path(file1.name))


class TestExecutableUtils(unittest.TestCase):
    def test_on_executable_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_executable_directory(dir1))

    def test_on_unexecutable_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            remove_execute_permission(dir1)
            self.assertFalse(is_executable_directory(dir1))

    def test_on_executable_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            add_execute_permission(file1.name)
            self.assertTrue(is_executable_file(file1.name))

    def test_on_unexecutable_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            remove_execute_permission(file1.name)
            self.assertFalse(is_executable_file(file1.name))

    def test_on_executable_path_as_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_executable_path(dir1))

    def test_on_unexecutable_path_as_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            remove_execute_permission(dir1)
            self.assertFalse(is_executable_path(dir1))

    def test_on_executable_path_as_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            add_execute_permission(file1.name)
            self.assertTrue(is_executable_path(file1.name))

    def test_on_unexecutable_path_as_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertFalse(is_executable_path(file1.name))


class TestSymbolicLinkUtils(unittest.TestCase):
    def setUp(self):
        # Create parent directory to hold symbolic links
        self.parent_directory = tempfile.mkdtemp()

        # Create temporary directory and a symbolic link pointing to it
        self.dir1 = tempfile.mkdtemp()
        self.link_to_dir1 = os.path.join(
            self.parent_directory, os.path.basename(self.dir1)
        )
        os.symlink(
            src=self.dir1, dst=self.link_to_dir1, target_is_directory=True
        )

        # Create temporary file and a symbolic link pointing to it
        self.file1 = tempfile.mkstemp()[1]
        self.link_to_file1 = os.path.join(
            self.parent_directory, os.path.basename(self.file1)
        )
        os.symlink(src=self.file1, dst=self.link_to_file1)

    def tearDown(self):
        # Unlink symbolic links
        os.unlink(self.link_to_file1)
        os.unlink(self.link_to_dir1)

        # Remove temporary directories and file
        shutil.rmtree(self.parent_directory)
        os.rmdir(self.dir1)
        os.remove(self.file1)

    def test_on_existing_link_to_directory(self):
        self.assertTrue(is_symbolic_link(self.link_to_dir1))

    def test_on_existing_link_to_file(self):
        self.assertTrue(is_symbolic_link(self.link_to_file1))

    def test_on_existing_directory(self):
        with tempfile.TemporaryDirectory() as temporary_directory:
            self.assertFalse(is_symbolic_link(temporary_directory))

    def test_on_existing_file(self):
        with tempfile.NamedTemporaryFile() as temporary_file:
            self.assertFalse(is_symbolic_link(temporary_file.name))


class TestResolvePath(unittest.TestCase):
    def test_on_directory(self):
        with tempfile.TemporaryDirectory() as dir1:
            expected = os.path.realpath(dir1)
            self.assertEqual(resolve_path(dir1), expected)

    def test_on_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            expected = os.path.realpath(file1.name)
            self.assertEqual(resolve_path(file1.name), expected)


class TestCreatePath(unittest.TestCase):
    def test_create_directory(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a directory in termporary directory
            dir1 = os.path.join(parent_directory, "SOMEDIR")
            create_directory(dir1)

            # Check if directory was created successfully
            self.assertTrue(os.path.isdir(dir1), True)

    def test_create_file(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file in termporary directory
            file1 = os.path.join(parent_directory, "SOMEFILE")
            create_file(file1)

            # Check if file was created successfully
            self.assertTrue(os.path.isfile(file1), True)


class TestExistingOrCreatablePath(unittest.TestCase):
    def test_on_existing_dir(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_existing_or_creatable_path(dir1))

    def test_on_nonexisting_creatable_dir(self):
        # Create and delete a temp directory so we know that it is a valid path
        dir1 = tempfile.mkdtemp()
        os.rmdir(dir1)
        self.assertTrue(is_existing_or_creatable_path(dir1))

    def test_on_uncreatable_dir(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a directory name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            directory_name = "SOMEDIR{}".format(forbidden_char)
            directory_path = os.path.join(parent_directory, directory_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_existing_or_creatable_path(directory_path))

    def test_on_existing_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_existing_or_creatable_path(file1.name))

    def test_on_nonexisting_creatable_file(self):
        # Create and delete a temp file so we know that it is a valid path
        file1 = tempfile.mkstemp()[1]
        os.remove(file1)
        self.assertTrue(is_existing_or_creatable_path(file1))

    def test_on_uncreatable_file(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            file_name = "SOMEDIR{}".format(forbidden_char)
            file_path = os.path.join(parent_directory, file_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_existing_or_creatable_path(file_path))


class TestValidPath(unittest.TestCase):
    def test_on_existing_dir(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_valid_path(dir1))

    def test_on_nonexisting_creatable_dir(self):
        # Create and delete a temp directory so we know that it is a valid path
        dir1 = tempfile.mkdtemp()
        os.rmdir(dir1)
        self.assertTrue(is_valid_path(dir1))

    def test_on_uncreatable_dir(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a directory name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            directory_name = "SOMEDIR{}".format(forbidden_char)
            directory_path = os.path.join(parent_directory, directory_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_path(directory_path))

    def test_on_existing_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_valid_path(file1.name))

    def test_on_nonexisting_creatable_file(self):
        # Create and delete a temp file so we know that it is a valid path
        file1 = tempfile.mkstemp()[1]
        os.remove(file1)
        self.assertTrue(is_valid_path(file1))

    def test_on_uncreatable_file(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            file_name = "SOMEDIR{}".format(forbidden_char)
            file_path = os.path.join(parent_directory, file_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_path(file_path))


class TestValidDirectory(unittest.TestCase):
    def test_on_existing_dir(self):
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_valid_directory(dir1))

    def test_on_nonexisting_creatable_dir(self):
        # Create and delete a temp directory so we know that it is a valid path
        dir1 = tempfile.mkdtemp()
        os.rmdir(dir1)
        self.assertTrue(is_valid_directory(dir1))

    def test_on_uncreatable_dir(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a directory name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            directory_name = "SOMEDIR{}".format(forbidden_char)
            directory_path = os.path.join(parent_directory, directory_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_directory(directory_path))


class TestValidFile(unittest.TestCase):
    def test_on_existing_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_valid_file(file1.name))

    def test_on_nonexisting_creatable_file(self):
        # Create and delete a temp file so we know that it is a valid path
        file1 = tempfile.mkstemp()[1]
        os.remove(file1)
        self.assertTrue(is_valid_file(file1))

    def test_on_uncreatable_file(self):
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            file_name = "SOMEDIR{}".format(forbidden_char)
            file_path = os.path.join(parent_directory, file_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_file(file_path))


class TestFileExtension(unittest.TestCase):
    def test_on_extension(self):
        with tempfile.NamedTemporaryFile(suffix=".EXT") as file1:
            self.assertEqual(get_extension(file1.name), "EXT")

    def test_on_no_extension(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertEqual(get_extension(file1.name), "")

    def test_on_two_word_extension(self):
        with tempfile.NamedTemporaryFile(suffix=".tar.gz") as file1:
            self.assertEqual(get_extension(file1.name), "gz")


class TestFileIsEmpty(unittest.TestCase):
    def test_on_empty_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_empty_file(file1.name))

    def test_on_nonempty_file(self):
        with tempfile.NamedTemporaryFile() as file1:
            with open(file1.name, "a") as file_for_writing:
                file_for_writing.write("SOME TEXT")
            self.assertFalse(is_empty_file(file1.name))



import ipaddress

import requests

__all__ = [
    "is_reachable_url",
    "is_valid_ip_address",
    "is_valid_ipv4_address",
    "is_valid_ipv6_address",
    "status_code_from_response_to_request_url",
]


def is_valid_ipv4_address(ip):
    """Return True if valid ipv4 address """

    try:
        ipaddress.IPv4Address(ip)
        return True
    except ipaddress.AddressValueError:
        return False


def is_valid_ipv6_address(ip):
    """Return True if valid ipv6 address """

    try:
        ipaddress.IPv6Address(ip)
        return True
    except ipaddress.AddressValueError:
        return False


def is_valid_ip_address(ip):
    """Return True if valid ipv4 or ipv6 address """
    return is_valid_ipv4_address(ip) or is_valid_ipv6_address(ip)


def is_reachable_url(url):
    """Return True if url is reachable"""

    try:
        # raise_for_status() raises an exception on fail, else None
        requests.get(url).raise_for_status()
        return True

    except requests.exceptions.RequestException:
        return False


def status_code_from_response_to_request_url(url):
    """Return status code from response to request url"""

    try:
        return str(requests.get(url).status_code)

    except requests.exceptions.RequestException:
        return None



import unittest

from action_hero.email_utils import is_valid_email


class TestIsEmailValid(unittest.TestCase):
    def test_on_valid_email(self):
        emails = ["hello@domain.com", "jobs@mac.com", "contact3@example.com"]
        [self.assertTrue(is_valid_email(email)) for email in emails]

    def test_on_invalid_email(self):
        emails = ["hello", "downcase", "2"]
        [self.assertFalse(is_valid_email(email)) for email in emails]



from action_hero.email import EmailIsValidAction
from action_hero.utils import ActionHeroTestCase


class TestEmailIsValidAction(ActionHeroTestCase):
    def test_on_valid_email(self):
        self.parser.add_argument("--email", action=EmailIsValidAction)
        email = "hello@service.com"
        # Parse without raising any errors
        self.parser.parse_args(["--email", email])

    def test_on_valid_emails_list(self):
        self.parser.add_argument(
            "--email", nargs="+", action=EmailIsValidAction
        )
        emails = ["hello@service.co.uk", "shawn@psych.com", "gus@psych.com"]
        # Parse without raising any errors
        self.parser.parse_args(["--email", *emails])

    def test_on_invalid_email(self):
        self.parser.add_argument("--email", action=EmailIsValidAction)
        email = "ampersand"
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--email", email])

    def test_on_invalid_emails_list(self):
        self.parser.add_argument(
            "--email", nargs="+", action=EmailIsValidAction
        )
        emails = ["friday", "person aol.com", "oopsemail"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--email", *emails])

    def test_on_mixed_valid_and_invalid_email_lists(self):
        self.parser.add_argument(
            "--email", nargs="+", action=EmailIsValidAction
        )
        emails = [
            "hello@service.co.uk",
            "shawn@psych.com",
            "gus@psych.com",
            "friday",
            "person aol.com",
            "oopsemail",
        ]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--email", *emails])



import unittest


class TestAll(unittest.TestCase):
    def test_on_valid_module_email(self):
        from action_hero import EmailIsValidAction

    def test_on_valid_module_net(self):
        from action_hero import IPIsValidIPAddressAction

    def test_on_valid_module_path(self):
        from action_hero import DirectoryDoesNotExistAction

    def test_on_valid_module_types(self):
        from action_hero import IsConvertibleToFloatAction

    def test_on_nonexisting_module(self):
        with self.assertRaises(ImportError):
            from action_hero import NonexistingModule



import os
import tempfile

from action_hero.utils import ActionHeroTestCase
from action_hero.path import (
    DirectoryDoesNotExistAction,
    DirectoryExistsAction,
    DirectoryIsExecutableAction,
    DirectoryIsNotExecutableAction,
    DirectoryIsNotReadableAction,
    DirectoryIsNotWritableAction,
    DirectoryIsReadableAction,
    DirectoryIsValidAction,
    DirectoryIsWritableAction,
    EnsureDirectoryAction,
    EnsureFileAction,
    FileDoesNotExistAction,
    FileExistsAction,
    FileIsEmptyAction,
    FileIsExecutableAction,
    FileIsNotEmptyAction,
    FileIsNotExecutableAction,
    FileIsNotReadableAction,
    FileIsNotWritableAction,
    FileIsReadableAction,
    FileIsWritableAction,
    FilenameHasExtension,
    PathDoesNotExistsAction,
    PathExistsAction,
    PathIsExecutableAction,
    PathIsNotExecutableAction,
    PathIsNotReadableAction,
    PathIsNotWritableAction,
    PathIsReadableAction,
    PathIsValidAction,
    PathIsWritableAction,
    ResolvePathAction,
)
from action_hero.path_utils import (
    add_execute_permission,
    is_empty_file,
    is_executable_directory,
    is_executable_file,
    is_existing_directory,
    is_existing_file,
    is_existing_path,
    is_readable_directory,
    is_readable_file,
    is_readable_path,
    is_valid_directory,
    is_valid_file,
    is_valid_path,
    is_writable_file,
    remove_execute_permission,
    remove_read_permission,
    remove_write_permission,
    resolve_path,
)


class TestResolvePathAction(ActionHeroTestCase):
    def test_processes_path(self):
        self.parser.add_argument("--path", action=ResolvePathAction)
        with tempfile.TemporaryDirectory() as dir1:
            args = self.parser.parse_args(["--path", dir1])
            self.assertIn("path", args)

    def test_on_single_path(self):
        self.parser.add_argument("--path", action=ResolvePathAction)
        with tempfile.TemporaryDirectory() as dir1:
            args = self.parser.parse_args(["--path", dir1])
            expected = resolve_path(dir1)
            self.assertEqual(args.path, expected)

    def test_on_list_of_paths(self):
        self.parser.add_argument("--path", nargs="+", action=ResolvePathAction)

        # Create few temporary paths
        path1 = tempfile.mkdtemp()
        path2 = tempfile.mkdtemp()
        path3 = tempfile.mkdtemp()
        temp_paths = [path1, path2, path3]

        # Parse args with list of paths
        args = self.parser.parse_args(["--path", *temp_paths])
        expected = [resolve_path(path) for path in temp_paths]
        self.assertEqual(args.path, expected)

        # Delete all temporary paths
        [os.rmdir(path) for path in temp_paths]


class TestEnsureDirectoryAction(ActionHeroTestCase):
    def test_on_nonexisting_directory(self):
        self.parser.add_argument("--path", action=EnsureDirectoryAction)
        with tempfile.TemporaryDirectory() as parent_directory:
            # Specify unique directory name
            dir1 = os.path.join(parent_directory, "NEWDIRT")
            # Assert specified directory does not exist
            self.assertFalse(os.path.isdir(dir1))
            # Parse args with --path as specified directory
            self.parser.parse_args(["--path", dir1])
            # Assert specified directory does exist
            self.assertTrue(os.path.isdir(dir1))

    def test_on_existing_directory(self):
        self.parser.add_argument("--path", action=EnsureDirectoryAction)
        # Specify directory to check
        with tempfile.TemporaryDirectory() as dir1:
            # Assert specified directory exists
            self.assertTrue(os.path.isdir(dir1))
            # Parse args with --path as specified directory
            self.parser.parse_args(["--path", dir1])
            # Assert specified directory still exist
            self.assertTrue(os.path.isdir(dir1))

    def test_on_multiple_mixed_existing_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=EnsureDirectoryAction
        )

        # Specify few new temporary directories
        dir1 = tempfile.mkdtemp()
        dir2 = tempfile.mkdtemp()

        # Directories deleted immediately to confirm they do not exist
        dir3 = tempfile.mkdtemp()
        os.rmdir(dir3)
        dir4 = tempfile.mkdtemp()
        os.rmdir(dir4)

        mixed_dirs = [dir1, dir2, dir3, dir4]

        # Assert that mixed_dirs contain existing and non-existing directories
        self.assertIn(True, [os.path.isdir(d) for d in mixed_dirs])
        self.assertIn(False, [os.path.isdir(d) for d in mixed_dirs])

        # Parse args with --path as specified file that does not exist
        self.parser.parse_args(["--path", *mixed_dirs])

        # Assert that all directories in mixed_dirs now exist
        self.assertNotIn(False, [os.path.isdir(d) for d in mixed_dirs])

        # Tear down temporary directories
        [os.rmdir(d) for d in mixed_dirs]


class TestEnsureFileAction(ActionHeroTestCase):
    def test_on_nonexisting_file(self):
        self.parser.add_argument("--path", action=EnsureFileAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify a file to check
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert specified  file exists
            self.assertTrue(os.path.isfile(file1))
            # Remove file
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(os.path.isfile(file1))

            # Parse args with --path as specified file that does not exist
            self.parser.parse_args(["--path", file1])
            # Assert specified file now exists
            self.assertTrue(os.path.isfile(file1))

    def test_on_existing_file(self):
        self.parser.add_argument("--path", action=EnsureFileAction)
        # Specify file to check
        with tempfile.NamedTemporaryFile() as file1:
            # Assert specified file exists
            self.assertTrue(os.path.isfile(file1.name))
            # Parse args with --path as specified file
            self.parser.parse_args(["--path", file1.name])
            # Assert specified file still exists
            self.assertTrue(os.path.isfile(file1.name))

    def test_on_multiple_mixed_existing_files(self):
        self.parser.add_argument("--path", nargs="+", action=EnsureFileAction)
        with tempfile.TemporaryDirectory() as parent_directory:
            # Specify few new temporary files
            file1 = tempfile.mkstemp(dir=parent_directory)[1]
            file2 = tempfile.mkstemp(dir=parent_directory)[1]

            # files deleted immediately to confirm they do not exist
            file3 = tempfile.mkstemp(dir=parent_directory)[1]
            os.remove(file3)
            file4 = tempfile.mkstemp(dir=parent_directory)[1]
            os.remove(file4)

            mixed_files = [file1, file2, file3, file4]

            # Assert that mixed_files contain existing and non-existing files
            self.assertIn(True, [os.path.isfile(f) for f in mixed_files])
            self.assertIn(False, [os.path.isfile(f) for f in mixed_files])

            # Parse args with --path as specified file that does not exist
            self.parser.parse_args(["--path", *mixed_files])

            # Assert that all files in mixed_files now exist
            self.assertNotIn(False, [os.path.isfile(f) for f in mixed_files])

            # Tear down temporary files
            [os.remove(f) for f in mixed_files]


class TestPathIsValidAction(ActionHeroTestCase):
    def test_on_valid_path(self):
        self.parser.add_argument("--path", action=PathIsValidAction)
        with tempfile.TemporaryDirectory() as path1:
            # Assert path is valid path
            self.assertTrue(is_valid_path(path1))
            # Parse args with list of paths
            args = self.parser.parse_args(["--path", path1])
            # Assert path from args is valid path
            self.assertTrue(is_valid_path(args.path))

    def test_on_multiple_valid_paths(self):
        self.parser.add_argument("--path", nargs="+", action=PathIsValidAction)

        # Create few temporary paths
        path1 = tempfile.mkdtemp()
        path2 = tempfile.mkdtemp()
        path3 = tempfile.mkdtemp()
        paths = [path1, path2, path3]

        # Parse args with list of paths
        self.parser.parse_args(["--path", *paths])
        self.assertNotIn(False, [is_valid_path(p) for p in paths])

        # Delete all temporary paths
        [os.rmdir(path) for path in paths]

    def test_on_invalid_path(self):
        self.parser.add_argument("--path", action=PathIsValidAction)
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            file_name = "SOMEFILE{}".format(forbidden_char)
            file_path = os.path.join(parent_directory, file_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_path(file_path))

            with self.assertRaises(ValueError):
                # Parse args with prohibited filename
                self.parser.parse_args(["--path", file_path])

    def test_on_mixed_valid_and_invalid_path(self):
        self.parser.add_argument("--path", nargs="+", action=PathIsValidAction)

        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            invalid_path_name = "SOMEFILE{}".format(forbidden_char)
            invalid_path = os.path.join(parent_directory, invalid_path_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_path(invalid_path))

            # Create valid path
            with tempfile.TemporaryDirectory() as valid_dir_path:

                # Assemble mixed valid and invalid path
                paths = [invalid_path, valid_dir_path]

                with self.assertRaises(ValueError):
                    # Parse args with list of paths
                    self.parser.parse_args(["--path", *paths])


class TestFileIsValidAction(ActionHeroTestCase):
    def test_on_valid_file_path(self):
        self.parser.add_argument("--path", action=PathIsValidAction)
        with tempfile.NamedTemporaryFile() as file_path:
            # Assert file path is valid file path
            self.assertTrue(is_valid_file(file_path.name))
            # Parse args with list of paths
            args = self.parser.parse_args(["--path", file_path.name])
            # Assert path from args is valid path
            self.assertTrue(is_valid_file(args.path))

    def test_on_multiple_valid_file_paths(self):
        self.parser.add_argument("--path", nargs="+", action=PathIsValidAction)

        # Create few temporary paths
        file_path1 = tempfile.mkstemp()[1]
        file_path2 = tempfile.mkstemp()[1]
        file_path3 = tempfile.mkstemp()[1]
        file_paths = [file_path1, file_path2, file_path3]

        # Parse args with list of paths
        self.parser.parse_args(["--path", *file_paths])
        self.assertNotIn(False, [is_valid_path(p) for p in file_paths])

        # Delete all temporary file paths
        [os.remove(path) for path in file_paths]

    def test_on_invalid_file_path(self):
        self.parser.add_argument("--path", action=PathIsValidAction)
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            file_name = "SOMEFILE{}".format(forbidden_char)
            file_path = os.path.join(parent_directory, file_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_file(file_path))

            with self.assertRaises(ValueError):
                # Parse args with prohibited filename
                self.parser.parse_args(["--path", file_path])

    def test_on_mixed_valid_and_invalid_file_path(self):
        self.parser.add_argument("--path", nargs="+", action=PathIsValidAction)

        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            invalid_file_name = "SOMEFILE{}".format(forbidden_char)
            invalid_file_path = os.path.join(
                parent_directory, invalid_file_name
            )

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_file(invalid_file_path))

            # Create valid path
            with tempfile.NamedTemporaryFile() as valid_file_path:

                # Assemble mixed valid and invalid path
                paths = [invalid_file_path, valid_file_path.name]

                with self.assertRaises(ValueError):
                    # Parse args with list of paths
                    self.parser.parse_args(["--path", *paths])


class TestDirectoryIsValidAction(ActionHeroTestCase):
    def test_on_valid_directory_path(self):
        self.parser.add_argument("--path", action=DirectoryIsValidAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Assert path is valid path
            self.assertTrue(is_valid_directory(dir1))
            # Parse args with list of paths
            args = self.parser.parse_args(["--path", dir1])
            # Assert path from args is valid path
            self.assertTrue(is_valid_directory(args.path))

    def test_on_multiple_valid_directory_paths(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryIsValidAction
        )

        # Create few temporary paths
        dir1 = tempfile.mkdtemp()
        dir2 = tempfile.mkdtemp()
        dir3 = tempfile.mkdtemp()
        dirs = [dir1, dir2, dir3]

        # Parse args with list of paths
        self.parser.parse_args(["--path", *dirs])
        self.assertNotIn(False, [is_valid_directory(d) for d in dirs])

        # Delete all temporary file paths
        [os.rmdir(d) for d in dirs]

    def test_on_invalid_directory_path(self):
        self.parser.add_argument("--path", action=PathIsValidAction)
        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            dir_name = "SOMEFILE{}".format(forbidden_char)
            dir_path = os.path.join(parent_directory, dir_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_file(dir_path))

            with self.assertRaises(ValueError):
                # Parse args with prohibited filename
                self.parser.parse_args(["--path", dir_path])

    def test_on_mixed_valid_and_invalid_file_path(self):
        self.parser.add_argument("--path", nargs="+", action=PathIsValidAction)

        with tempfile.TemporaryDirectory() as parent_directory:
            # Create a file name with a char forbidden in POSIX and WIN*
            forbidden_char = "/"
            invalid_dir_name = "SOMEDIR{}".format(forbidden_char)
            invalid_dir_path = os.path.join(parent_directory, invalid_dir_name)

            # Assert that the forbidden character prohibited path creation
            self.assertFalse(is_valid_file(invalid_dir_path))

            # Create valid path
            with tempfile.TemporaryDirectory() as valid_dir_path:

                # Assemble mixed valid and invalid path
                paths = [invalid_dir_path, valid_dir_path]

                with self.assertRaises(ValueError):
                    # Parse args with list of paths
                    self.parser.parse_args(["--path", *paths])


class TestPathExistsAction(ActionHeroTestCase):
    def test_on_existing_path(self):
        self.parser.add_argument("--path", action=PathExistsAction)
        # Specify file to check
        with tempfile.NamedTemporaryFile() as file1:
            # Assert specified file exists
            self.assertTrue(is_existing_path(file1.name))
            # Parse args with --path as specified file
            self.parser.parse_args(["--path", file1.name])
            # Assert specified file still exists
            self.assertTrue(is_existing_path(file1.name))

    def test_on_nonexisting_path(self):
        self.parser.add_argument("--path", action=PathExistsAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify a file to check
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert specified  file exists
            self.assertTrue(is_existing_path(file1))
            # Remove file
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(is_existing_path(file1))

            with self.assertRaises(ValueError):
                # Parse args with --path as specified file that does not exist
                self.parser.parse_args(["--path", file1])

    def test_on_mixed_existing_and_nonexisting_path(self):
        self.parser.add_argument("--path", nargs="+", action=PathExistsAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify a file to check
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert specified  file exists
            self.assertTrue(is_existing_path(file1))
            # Remove file
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(is_existing_path(file1))
            # Assemble mixed list of existing and nonexisting paths
            paths = [dir1, file1]

            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", *paths])


class TestPathDoesNotExistsAction(ActionHeroTestCase):
    def test_on_existing_path(self):
        self.parser.add_argument("--path", action=PathDoesNotExistsAction)
        # Specify file to check
        with tempfile.NamedTemporaryFile() as file1:
            with self.assertRaises(ValueError):
                # Parse args with --path as specified file that does not exist
                self.parser.parse_args(["--path", file1.name])

    def test_on_nonexisting_path(self):
        self.parser.add_argument("--path", action=PathDoesNotExistsAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify a file to check
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert specified  file exists
            self.assertTrue(is_existing_path(file1))
            # Remove file
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(is_existing_path(file1))
            # Parse args with --path as specified file
            self.parser.parse_args(["--path", file1])
            # Assert specified file no longer exists
            self.assertFalse(is_existing_path(file1))

    def test_on_mixed_existing_and_nonexisting_path(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathDoesNotExistsAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify a file to check
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert specified  file exists
            self.assertTrue(is_existing_path(file1))
            # Remove file
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(is_existing_path(file1))
            # Assemble mixed list of existing and nonexisting paths
            paths = [dir1, file1]

            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", *paths])


class TestFileExistsAction(ActionHeroTestCase):
    def test_on_existing_path(self):
        self.parser.add_argument("--path", action=FileExistsAction)
        # Specify file to check
        with tempfile.NamedTemporaryFile() as file1:
            # Assert specified file exists
            self.assertTrue(is_existing_file(file1.name))
            # Parse args with --path as specified file
            self.parser.parse_args(["--path", file1.name])
            # Assert specified file still exists
            self.assertTrue(is_existing_file(file1.name))

    def test_on_nonexisting_file(self):
        self.parser.add_argument("--path", action=FileExistsAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify a file to check
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert specified  file exists
            self.assertTrue(is_existing_file(file1))
            # Remove file
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(is_existing_file(file1))

            with self.assertRaises(ValueError):
                # Parse args with --path as specified file that does not exist
                self.parser.parse_args(["--path", file1])

    def test_on_mixed_existing_and_nonexisting_path(self):
        self.parser.add_argument("--path", nargs="+", action=FileExistsAction)
        with tempfile.TemporaryDirectory() as directory:
            # Specify file to check
            file1 = tempfile.mkstemp(dir=directory)[1]
            file2 = tempfile.mkstemp(dir=directory)[1]
            file3 = tempfile.mkstemp(dir=directory)[1]
            # Assert specified files exists
            self.assertTrue(is_existing_file(file1))
            self.assertTrue(is_existing_file(file2))
            self.assertTrue(is_existing_file(file3))
            # Remove file 1
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(is_existing_file(file1))
            # Assemble mixed list of existing and nonexisting paths
            paths = [file1, file2, file3]

            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", *paths])


class TestFileDoesNotExistsAction(ActionHeroTestCase):
    def test_on_existing_path(self):
        self.parser.add_argument("--path", action=FileDoesNotExistAction)
        # Specify file to check
        with tempfile.NamedTemporaryFile() as file1:
            with self.assertRaises(ValueError):
                # Parse args with --path as specified file that does not exist
                self.parser.parse_args(["--path", file1.name])

    def test_on_nonexisting_path(self):
        self.parser.add_argument("--path", action=FileDoesNotExistAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify a file to check
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert specified  file exists
            self.assertTrue(is_existing_file(file1))
            # Remove file
            os.remove(file1)
            # Assert specified file no longer exists
            self.assertFalse(is_existing_file(file1))
            # Parse args with --path as specified file
            self.parser.parse_args(["--path", file1])
            # Assert specified file no longer exists
            self.assertFalse(is_existing_file(file1))

    def test_on_mixed_existing_and_nonexisting_path(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileDoesNotExistAction
        )
        # Specify a file to check
        file1 = tempfile.mkstemp()[1]
        file2 = tempfile.mkstemp()[1]
        # Assert specified file exists
        self.assertTrue(is_existing_file(file1))
        self.assertTrue(is_existing_file(file2))
        # Remove file
        os.remove(file1)
        self.assertFalse(is_existing_file(file1))
        # Assemble mixed list of existing and nonexisting paths
        paths = [file1, file2]

        with self.assertRaises(ValueError):
            self.parser.parse_args(["--path", *paths])

        # Tear down remaining temporary files
        os.remove(file2)


class TestDirectoryExistsAction(ActionHeroTestCase):
    def test_on_existing_path(self):
        self.parser.add_argument("--path", action=DirectoryExistsAction)
        # Specify directory to check
        with tempfile.TemporaryDirectory() as dir1:
            # Assert specified directory exists
            self.assertTrue(is_existing_directory(dir1))
            # Parse args with --path as specified directory
            args = self.parser.parse_args(["--path", dir1])
            # Assert specified directory still exists
            self.assertTrue(is_existing_path(dir1))
            self.assertTrue(is_existing_path(args.path))

    def test_on_nonexisting_path(self):
        self.parser.add_argument("--path", action=DirectoryExistsAction)
        # Specifiy directory
        dir1 = tempfile.mkdtemp()
        # Assert specified directory exists
        self.assertTrue(is_existing_directory(dir1))
        # Remove specified directory
        os.rmdir(dir1)
        # Assert specified directory doesnt exist
        self.assertFalse(is_existing_directory(dir1))

        with self.assertRaises(ValueError):
            # Parse args with list of paths
            self.parser.parse_args(["--path", dir1])

    def test_on_mixed_existing_and_nonexisting_path(self):
        self.parser.add_argument("--path", action=DirectoryExistsAction)
        # Specifiy directores
        dir1 = tempfile.mkdtemp()
        dir2 = tempfile.mkdtemp()
        # Assert specified directory exists
        self.assertTrue(is_existing_directory(dir1))
        self.assertTrue(is_existing_directory(dir2))
        # Remove specified directory
        os.rmdir(dir1)
        # Assert specified directory doesnt exist
        self.assertFalse(is_existing_directory(dir1))

        with self.assertRaises(ValueError):
            # Parse args with list of paths
            self.parser.parse_args(["--path", dir1, dir2])


class TestDirectoryDoesNotExistsAction(ActionHeroTestCase):
    def test_on_existing_path(self):
        self.parser.add_argument("--path", action=DirectoryDoesNotExistAction)
        with tempfile.TemporaryDirectory() as dir1:
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_nonexisting_path(self):
        self.parser.add_argument("--path", action=DirectoryDoesNotExistAction)
        # Specify directory
        dir1 = tempfile.mkdtemp()
        # Assert directory exists
        self.assertTrue(is_existing_directory(dir1))
        # Remove specified directory
        os.rmdir(dir1)
        # Pargs args with removed specified directory
        self.parser.parse_args(["--path", dir1])

    def test_on_mixed_existing_and_nonexisting_path(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryDoesNotExistAction
        )
        # Specify directories
        dir1 = tempfile.mkdtemp()
        dir2 = tempfile.mkdtemp()
        # Assert directories exists
        self.assertTrue(is_existing_directory(dir1))
        self.assertTrue(is_existing_directory(dir2))
        # Remove one specified directory
        os.rmdir(dir1)
        # Pargs args with one removed specified directory
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--path", dir1, dir2])
        # Tear down remaining temporary directories
        os.rmdir(dir2)


class TestFileIsWritableAction(ActionHeroTestCase):
    def test_on_writable_file(self):
        self.parser.add_argument("--path", action=FileIsWritableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert fil eis writable
            self.assertTrue(is_writable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])
            # Assert file is still writable
            self.assertTrue(is_writable_file(file1.name))

    def test_on_unwritable_file(self):
        self.parser.add_argument("--path", action=FileIsWritableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert fileis writable
            self.assertTrue(is_writable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])

    def test_on_mixed_writable_and_unwritable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileIsWritableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify writable and unwritable file
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            remove_write_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])


class TestFileIsNotWritableAction(ActionHeroTestCase):
    def test_on_writable_file(self):
        self.parser.add_argument("--path", action=FileIsNotWritableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert file is writable
            self.assertTrue(is_writable_file(file1.name))
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_unwritable_file(self):
        self.parser.add_argument("--path", action=FileIsNotWritableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify file and remove write permission
            file1 = tempfile.mkstemp(dir=dir1)[1]
            remove_write_permission(file1)
            # Assert file is unwritable
            self.assertFalse(is_writable_file(file1))
            # No Error on parse args
            self.parser.parse_args(["--path", file1])
            # Assert file is unwritable
            self.assertFalse(is_writable_file(file1))

    def test_on_mixed_writable_and_unwritable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileIsNotWritableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify writable and unwritable files
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            remove_write_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])


class TestDirectoryIsWritableAction(ActionHeroTestCase):
    def test_on_writable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsWritableAction)
        # Specify writable directory
        with tempfile.TemporaryDirectory() as dir1:
            # Parse with readable directory
            self.parser.parse_args(["--path", dir1])

    def test_on_wunwritable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsWritableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir2])

    def test_on_writable_and_unwritable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryIsWritableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestDirectoryIsNotWritableAction(ActionHeroTestCase):
    def test_on_writable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsNotWritableAction)
        # Specify writable directory
        with tempfile.TemporaryDirectory() as dir1:
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_unwritable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsNotWritableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Parse with readable directory
            self.parser.parse_args(["--path", dir2])

    def test_on_writable_and_unwritable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryIsNotWritableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestPathIsWritableAction(ActionHeroTestCase):
    def test_on_writable_directory(self):
        self.parser.add_argument("--path", action=PathIsWritableAction)
        # Specify writable directory
        with tempfile.TemporaryDirectory() as dir1:
            # Parse with readable directory
            self.parser.parse_args(["--path", dir1])

    def test_on_unwritable_directory(self):
        self.parser.add_argument("--path", action=PathIsWritableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir2])

    def test_on_writable_and_unwritable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsWritableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestPathIsNotWritableAction(ActionHeroTestCase):
    def test_on_writable_directory(self):
        self.parser.add_argument("--path", action=PathIsNotWritableAction)
        # Specify writable directory
        with tempfile.TemporaryDirectory() as dir1:
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_unwritable_directory(self):
        self.parser.add_argument("--path", action=PathIsNotWritableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Parse with readable directory
            self.parser.parse_args(["--path", dir2])

    def test_on_writable_and_unwritable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsNotWritableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify unwritable directory and remove write permissions
            dir2 = tempfile.mkdtemp(dir=dir1)
            remove_write_permission(dir2)
            # Asserts error on parsing unwritable directory
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestFileIsReadableAction(ActionHeroTestCase):
    def test_on_readable_file(self):
        self.parser.add_argument("--path", action=FileIsReadableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert file is readable
            self.assertTrue(is_readable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])
            # Assert file is still readable
            self.assertTrue(is_readable_file(file1.name))

    def test_on_unreadable_file(self):
        self.parser.add_argument("--path", action=FileIsReadableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert file is readable
            self.assertTrue(is_readable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])

    def test_on_mixed_readable_and_unreadable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileIsReadableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify readable and unreadable file
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            remove_read_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])


class TestFileIsNotReadableAction(ActionHeroTestCase):
    def test_on_readable_file(self):
        self.parser.add_argument("--path", action=FileIsNotReadableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert file is readable
            self.assertTrue(is_readable_file(file1.name))
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_unreadable_file(self):
        self.parser.add_argument("--path", action=FileIsNotReadableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify file and remove write permission
            file1 = tempfile.mkstemp(dir=dir1)[1]
            remove_read_permission(file1)
            # Assert file is unreadable
            self.assertFalse(is_readable_file(file1))
            # No Error on parse args
            self.parser.parse_args(["--path", file1])
            # Assert file is unreadable
            self.assertFalse(is_readable_file(file1))

    def test_on_mixed_readable_and_unreadable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileIsNotReadableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify readable and unreadable files
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            remove_write_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])


class TestDirectoryIsReadableAction(ActionHeroTestCase):
    def test_on_readable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsReadableAction)
        # Specify directory
        with tempfile.TemporaryDirectory() as dir1:
            # Assert directory is readable
            self.assertTrue(is_readable_directory(dir1))
            # No errors when parsing args
            self.parser.parse_args(["--path", dir1])
            # Assert directory is still readable
            self.assertTrue(is_readable_directory(dir1))

    def test_on_unreadable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsReadableAction)
        # Specify dir amd make unreadable
        dir1 = tempfile.mkdtemp()
        remove_read_permission(dir1)
        # Assert ValueError raised when parsing args
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--path", dir1])
        # Tear down temp dirs
        os.rmdir(dir1)

    def test_on_mixed_readable_and_unreadable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryIsReadableAction
        )
        # Specify readable and unreadable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_read_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestDirectoryIsNotReadableAction(ActionHeroTestCase):
    def test_on_readable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsNotReadableAction)
        # Specify directory
        with tempfile.TemporaryDirectory() as dir1:
            # Assert directory is readable
            self.assertTrue(is_readable_directory(dir1))
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_unreadable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsNotReadableAction)
        # Specify dir amd make unreadable
        dir1 = tempfile.mkdtemp()
        remove_read_permission(dir1)
        # No errors when parsing args
        self.parser.parse_args(["--path", dir1])
        # Assert directory is still unreadable
        self.assertFalse(is_readable_directory(dir1))
        # Tear down temp dirs
        os.rmdir(dir1)

    def test_on_mixed_readable_and_unreadable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryIsNotReadableAction
        )
        # Specify readable and unreadable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_read_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestPathIsReadableAction(ActionHeroTestCase):
    def test_on_readable_file(self):
        self.parser.add_argument("--path", action=PathIsReadableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert file is readable
            self.assertTrue(is_readable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])
            # Assert file is still readable
            self.assertTrue(is_readable_file(file1.name))

    def test_on_unreadable_file(self):
        self.parser.add_argument("--path", action=PathIsReadableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert file is readable
            self.assertTrue(is_readable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])

    def test_on_mixed_readable_and_unreadable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsReadableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify readable and unreadable file
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            remove_read_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])

    def test_on_readable_directory(self):
        self.parser.add_argument("--path", action=PathIsReadableAction)
        # Specify directory
        with tempfile.TemporaryDirectory() as dir1:
            # Assert directory is readable
            self.assertTrue(is_readable_directory(dir1))
            # No errors when parsing args
            self.parser.parse_args(["--path", dir1])
            # Assert directory is still readable
            self.assertTrue(is_readable_directory(dir1))

    def test_on_unreadable_directory(self):
        self.parser.add_argument("--path", action=PathIsReadableAction)
        # Specify dir amd make unreadable
        dir1 = tempfile.mkdtemp()
        remove_read_permission(dir1)
        # Assert ValueError raised when parsing args
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--path", dir1])
        # Tear down temp dirs
        os.rmdir(dir1)

    def test_on_mixed_readable_and_unreadable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsReadableAction
        )
        # Specify readable and unreadable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_read_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestPathIsNotReadableAction(ActionHeroTestCase):
    def test_on_readable_file(self):
        self.parser.add_argument("--path", action=PathIsNotReadableAction)
        # Specify file
        with tempfile.NamedTemporaryFile() as file1:
            # Assert file is readable
            self.assertTrue(is_readable_file(file1.name))
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_unreadable_file(self):
        self.parser.add_argument("--path", action=PathIsNotReadableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify file and remove write permission
            file1 = tempfile.mkstemp(dir=dir1)[1]
            remove_read_permission(file1)
            # Assert file is unreadable
            self.assertFalse(is_readable_file(file1))
            # No Error on parse args
            self.parser.parse_args(["--path", file1])
            # Assert file is unreadable
            self.assertFalse(is_readable_path(file1))

    def test_on_mixed_readable_and_unreadable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsNotReadableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify readable and unreadable files
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            remove_write_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])

    def test_on_readable_directory(self):
        self.parser.add_argument("--path", action=PathIsNotReadableAction)
        # Specify directory
        with tempfile.TemporaryDirectory() as dir1:
            # Assert directory is readable
            self.assertTrue(is_readable_directory(dir1))
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_unreadable_directory(self):
        self.parser.add_argument("--path", action=PathIsNotReadableAction)
        # Specify dir amd make unreadable
        dir1 = tempfile.mkdtemp()
        remove_read_permission(dir1)
        # No errors when parsing args
        self.parser.parse_args(["--path", dir1])
        # Assert directory is still unreadable
        self.assertFalse(is_readable_directory(dir1))
        # Tear down temp dirs
        os.rmdir(dir1)

    def test_on_mixed_readable_and_unreadable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsNotReadableAction
        )
        # Specify readable and unreadable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_read_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestFileIsExecutableAction(ActionHeroTestCase):
    def test_on_executable_file(self):
        self.parser.add_argument("--path", action=FileIsExecutableAction)
        # Specify file and make executable
        with tempfile.NamedTemporaryFile() as file1:
            add_execute_permission(file1.name)
            self.assertTrue(is_executable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])

    def test_on_unexecutable_file(self):
        self.parser.add_argument("--path", action=FileIsExecutableAction)
        # Specify file and assert not executable
        with tempfile.NamedTemporaryFile() as file1:
            self.assertFalse(is_executable_file(file1.name))
            # Assert ValueError on parsing
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_mixed_executable_and_unexecutable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileIsExecutableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify readable and unreadable file
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            add_execute_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])


class TestFileIsNotExecutableAction(ActionHeroTestCase):
    def test_on_executable_file(self):
        self.parser.add_argument("--path", action=FileIsNotExecutableAction)
        # Specify file and make executable
        with tempfile.NamedTemporaryFile() as file1:
            add_execute_permission(file1.name)
            self.assertTrue(is_executable_file(file1.name))
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_unreadable_file(self):
        self.parser.add_argument("--path", action=FileIsNotExecutableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify file and remove write permission
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert file is unexecutable
            self.assertFalse(is_executable_file(file1))
            # No Error on parse args
            self.parser.parse_args(["--path", file1])
            # Assert file is unexecutable
            self.assertFalse(is_executable_file(file1))

    def test_on_mixed_executable_and_unexecutable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileIsNotExecutableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify executable and unexecutable files
            file1 = tempfile.mkstemp(dir=dir1)[1]
            add_execute_permission(file1)
            file2 = tempfile.mkstemp(dir=dir1)[1]
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])


class TestDirectoryIsExecutableAction(ActionHeroTestCase):
    def test_on_executable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsExecutableAction)
        # Specify directory and make executable
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_executable_directory(dir1))
            # No errors when parsing args
            self.parser.parse_args(["--path", dir1])

    def test_on_unexecutable_directory(self):
        self.parser.add_argument("--path", action=DirectoryIsExecutableAction)
        # Specify dir and make unexecutable
        with tempfile.TemporaryDirectory() as dir1:
            remove_execute_permission(dir1)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_mixed_executable_and_unexecutable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryIsExecutableAction
        )
        # Specify executable and unexecutable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_execute_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestDirectoryIsNotExecutableAction(ActionHeroTestCase):
    def test_on_executable_directory(self):
        self.parser.add_argument(
            "--path", action=DirectoryIsNotExecutableAction
        )
        # Specify directory
        with tempfile.TemporaryDirectory() as dir1:
            # Assert directory is executable
            self.assertTrue(is_executable_directory(dir1))
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_unexecutable_directory(self):
        self.parser.add_argument(
            "--path", action=DirectoryIsNotExecutableAction
        )
        # Specify dir amd make unexecutable
        with tempfile.TemporaryDirectory() as dir1:
            dir1 = tempfile.mkdtemp()
            remove_execute_permission(dir1)
            # No errors when parsing args
            self.parser.parse_args(["--path", dir1])
            # Assert directory is still unreadable
            self.assertFalse(is_executable_directory(dir1))

    def test_on_mixed_executable_and_unexecutable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=DirectoryIsNotExecutableAction
        )
        # Specify executable and unexecutable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_execute_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestPathIsExecutableAction(ActionHeroTestCase):
    def test_on_executable_file(self):
        self.parser.add_argument("--path", action=PathIsExecutableAction)
        # Specify file and make executable
        with tempfile.NamedTemporaryFile() as file1:
            add_execute_permission(file1.name)
            self.assertTrue(is_executable_file(file1.name))
            # No errors when parsing args
            self.parser.parse_args(["--path", file1.name])

    def test_on_unexecutable_file(self):
        self.parser.add_argument("--path", action=PathIsExecutableAction)
        # Specify file and assert not executable
        with tempfile.NamedTemporaryFile() as file1:
            self.assertFalse(is_executable_file(file1.name))
            # Assert ValueError on parsing
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_mixed_executable_and_unexecutable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsExecutableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify readable and unreadable file
            file1 = tempfile.mkstemp(dir=dir1)[1]
            file2 = tempfile.mkstemp(dir=dir1)[1]
            add_execute_permission(file2)
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])

    def test_on_executable_directory(self):
        self.parser.add_argument("--path", action=PathIsExecutableAction)
        # Specify directory and make executable
        with tempfile.TemporaryDirectory() as dir1:
            self.assertTrue(is_executable_directory(dir1))
            # No errors when parsing args
            self.parser.parse_args(["--path", dir1])

    def test_on_unexecutable_directory(self):
        self.parser.add_argument("--path", action=PathIsExecutableAction)
        # Specify dir and make unexecutable
        with tempfile.TemporaryDirectory() as dir1:
            remove_execute_permission(dir1)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_mixed_executable_and_unexecutable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsExecutableAction
        )
        # Specify executable and unexecutable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_execute_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestPathIsNotExecutableAction(ActionHeroTestCase):
    def test_on_executable_file(self):
        self.parser.add_argument("--path", action=PathIsNotExecutableAction)
        # Specify file and make executable
        with tempfile.NamedTemporaryFile() as file1:
            add_execute_permission(file1.name)
            self.assertTrue(is_executable_file(file1.name))
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_unreadable_file(self):
        self.parser.add_argument("--path", action=PathIsNotExecutableAction)
        with tempfile.TemporaryDirectory() as dir1:
            # Specify file and remove write permission
            file1 = tempfile.mkstemp(dir=dir1)[1]
            # Assert file is unexecutable
            self.assertFalse(is_executable_file(file1))
            # No Error on parse args
            self.parser.parse_args(["--path", file1])
            # Assert file is unexecutable
            self.assertFalse(is_executable_file(file1))

    def test_on_mixed_executable_and_unexecutable_file(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsNotExecutableAction
        )
        with tempfile.TemporaryDirectory() as dir1:
            # Specify executable and unexecutable files
            file1 = tempfile.mkstemp(dir=dir1)[1]
            add_execute_permission(file1)
            file2 = tempfile.mkstemp(dir=dir1)[1]
            # Check if ValueError raised on parse
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1, file2])

    def test_on_executable_directory(self):
        self.parser.add_argument("--path", action=PathIsNotExecutableAction)
        # Specify directory
        with tempfile.TemporaryDirectory() as dir1:
            # Assert directory is executable
            self.assertTrue(is_executable_directory(dir1))
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1])

    def test_on_unexecutable_directory(self):
        self.parser.add_argument("--path", action=PathIsNotExecutableAction)
        # Specify dir amd make unexecutable
        with tempfile.TemporaryDirectory() as dir1:
            dir1 = tempfile.mkdtemp()
            remove_execute_permission(dir1)
            # No errors when parsing args
            self.parser.parse_args(["--path", dir1])
            # Assert directory is still unreadable
            self.assertFalse(is_executable_directory(dir1))

    def test_on_mixed_executable_and_unexecutable_directories(self):
        self.parser.add_argument(
            "--path", nargs="+", action=PathIsNotExecutableAction
        )
        # Specify executable and unexecutable dirs
        with tempfile.TemporaryDirectory() as dir1:
            dir2 = tempfile.mkdtemp()
            remove_execute_permission(dir2)
            # Assert ValueError raised when parsing args
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", dir1, dir2])


class TestFileIsEmptyAction(ActionHeroTestCase):
    def test_on_empty_file(self):
        self.parser.add_argument("--path", action=FileIsEmptyAction)
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_empty_file(file1.name))
            self.parser.parse_args(["--path", file1.name])

    def test_on_nonempty_file(self):
        self.parser.add_argument("--path", action=FileIsEmptyAction)
        with tempfile.NamedTemporaryFile() as file1:
            with open(file1.name, "a") as file_for_writing:
                file_for_writing.write("SOME TEXT")
            self.assertFalse(is_empty_file(file1.name))
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_list_of_empty_and_nonempty_files(self):
        self.parser.add_argument("--path", nargs="+", action=FileIsEmptyAction)
        with tempfile.NamedTemporaryFile() as file1:
            with tempfile.NamedTemporaryFile() as file2:
                with open(file2.name, "a") as file_for_writing:
                    file_for_writing.write("SOME TEXT")
                self.assertTrue(is_empty_file(file1.name))
                self.assertFalse(is_empty_file(file2.name))
                with self.assertRaises(ValueError):
                    self.parser.parse_args(["--path", file1.name, file2.name])


class TestFileIsNotEmptyAction(ActionHeroTestCase):
    def test_on_empty_file(self):
        self.parser.add_argument("--path", action=FileIsNotEmptyAction)
        with tempfile.NamedTemporaryFile() as file1:
            self.assertTrue(is_empty_file(file1.name))
            with self.assertRaises(ValueError):
                self.parser.parse_args(["--path", file1.name])

    def test_on_nonempty_file(self):
        self.parser.add_argument("--path", action=FileIsNotEmptyAction)
        with tempfile.NamedTemporaryFile() as file1:
            with open(file1.name, "a") as file_for_writing:
                file_for_writing.write("SOME TEXT")
            self.assertFalse(is_empty_file(file1.name))
            self.parser.parse_args(["--path", file1.name])

    def test_on_list_of_empty_and_nonempty_files(self):
        self.parser.add_argument(
            "--path", nargs="+", action=FileIsNotEmptyAction
        )
        with tempfile.NamedTemporaryFile() as file1:
            with tempfile.NamedTemporaryFile() as file2:
                with open(file2.name, "a") as file_for_writing:
                    file_for_writing.write("SOME TEXT")
                self.assertTrue(is_empty_file(file1.name))
                self.assertFalse(is_empty_file(file2.name))
                with self.assertRaises(ValueError):
                    self.parser.parse_args(["--path", file1.name, file2.name])


class TestFilenameHasExtension(ActionHeroTestCase):
    def test_on_parser_with_extension(self):
        self.parser.add_argument(
            "--filename", action=FilenameHasExtension, action_values=["txt"]
        )

    def test_on_parser_without_extension(self):
        with self.assertRaises(ValueError):
            self.parser.add_argument("--filename", action=FilenameHasExtension)

    def test_on_filename_with_matching_extension(self):
        self.parser.add_argument(
            "--filename", action=FilenameHasExtension, action_values=["txt"]
        )
        self.parser.parse_args(["--filename", "diary.txt"])

    def test_on_filename_with_nonmatching_extension(self):
        self.parser.add_argument(
            "--filename", action=FilenameHasExtension, action_values=["txt"]
        )
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--filename", "diary.md"])

    def test_on_list_of_filenames_with_matching_extension(self):
        self.parser.add_argument(
            "--filename",
            nargs="+",
            action=FilenameHasExtension,
            action_values=["txt"],
        )
        self.parser.parse_args(
            ["--filename", "diary.txt", "log.txt", "lyrics.txt"]
        )

    def test_on_list_of_filenames_with_nonmatching_extension(self):
        self.parser.add_argument(
            "--filename",
            nargs="+",
            action=FilenameHasExtension,
            action_values=["txt"],
        )
        with self.assertRaises(ValueError):
            self.parser.parse_args(
                ["--filename", "diary.md", "README.rst", "history.sh"]
            )

    def test_on_list_of_filenames_with_mixed_matching_extensions(self):
        self.parser.add_argument(
            "--filename",
            nargs="+",
            action=FilenameHasExtension,
            action_values=["txt"],
        )
        with self.assertRaises(ValueError):
            self.parser.parse_args(
                [
                    "--filename",
                    "notes.txt",
                    "diary.md",
                    "README.rst",
                    "history.sh",
                ]
            )

    def test_on_multiple_action_values(self):
        self.parser.add_argument(
            "--filename",
            action=FilenameHasExtension,
            action_values=["md", "markdown"],
        )
        self.parser.parse_args(["--filename", "diary.md"])

    def test_on_multiple_action_values_with_expected_filenames(self):
        self.parser.add_argument(
            "--filename",
            action=FilenameHasExtension,
            action_values=["md", "markdown"],
        )
        with self.assertRaises(ValueError):
            self.parser.parse_args(
                ["--filename", "README.md", "blogentry.markdown"]
            )

    def test_on_multiple_action_values_with_unexpected_filenames(self):
        self.parser.add_argument(
            "--filename",
            action=FilenameHasExtension,
            action_values=["md", "markdown"],
        )
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--filename", "config.yml"])

    def test_on_multiple_action_values_with_mixed_expected_filenames(self):
        self.parser.add_argument(
            "--filename",
            nargs="+",
            action=FilenameHasExtension,
            action_values=["md", "markdown"],
        )
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--filename", "config.yml", "README.rst"])



import sys
import unittest
import functools
import argparse

import requests


__all__ = [
    "ActionHeroAction",
    "CheckAction",
    "CheckPresentInValuesAction",
    "MapAction",
    "MapAndReplaceAction",
    "PipelineAction",
    "run_only_when_when_internet_is_up",
    "run_only_when_modules_loaded",
]


def run_only_when_modules_loaded(modules=["argparse"]):
    """Decorator that runs wrapped function only when the supplied modules are
    loaded

    Args:
        modules (list[srtr]): List of modules to check if loaded

    """

    def run_only_when_modules_loaded_wrapper(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            # Only run func if module loaded
            if all([(module in sys.modules) for module in modules]):
                func(*args, **kwargs)

        return wrapper

    return run_only_when_modules_loaded_wrapper


def run_only_when_when_internet_is_up(urls=["http://www.google.com"]):
    """Decorator that runs wrapped function when the internet is up.

    Connection is checked by checking connection to values in urls

    Args:
        urls (list[str]): List of urls to check for when checking for
        connection

    """

    def run_only_when_when_internet_is_up_wrapper(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            # Do network check
            try:
                [requests.get(url).raise_for_status() for url in urls]

                func(*args, **kwargs)

            # Do nothing on response error
            except requests.exceptions.RequestException:
                pass

        return wrapper

    return run_only_when_when_internet_is_up_wrapper


class ActionHeroAction(argparse.Action):
    pass


class BaseAction(ActionHeroAction):
    """ArgumentParser Action subclass that runs user's func over values

    ActionHero' Baseclass for subclasses that need access to a func,
        err_msg_singular and err_msg_plural

    Attributes:
        func (func): To be used to fill in subclasses preferred func.
        err_msg_singular(str): To be used to fill error message for singular
            failure.
        err_msg_plural(str): To be used to fill error message for singular
            failure.

    """

    func = None
    err_msg_singular = None
    err_msg_plural = None

    @classmethod
    def run_user_func(cls, value):
        """Runs cls.func over value

        Used nside __call__

        Needs to be @classmethod to avoid
        1. including self when being called in other methods
        2. not including self when calling other funcs within

        Args:
            cls (cls): classmethod argument
            value (type): The value to run cls.func upon

        """
        return cls.func(value)


class CheckAction(BaseAction):
    """Checks all values return True with func. Args from superclass
    argparse.Action

    """

    def __init__(
        self, option_strings, dest, nargs=None, help=None, metavar=None
    ):
        for attr in ["func", "err_msg_singular", "err_msg_plural"]:
            # Use getattr. hasattr returns True as they're initialized to None.
            if not getattr(self, attr):
                raise ValueError(
                    "Please supply required attribute: {}".format(attr)
                )

        super(CheckAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            help=help,
            metavar=metavar,
        )

    def __call__(self, parser, namespace, values, option_string=None):
        # When values are a list of strings
        if isinstance(values, list):
            if not all([self.run_user_func(value) for value in values]):
                raise argparse.ArgumentError(self, self.err_msg_plural)

        # When values is one string
        else:
            value = values
            if not self.run_user_func(value):
                raise argparse.ArgumentError(self, self.err_msg_singular)

        setattr(namespace, self.dest, values)


class CheckPresentInValuesAction(BaseAction):
    """Checks result func over each value in values is in action_values.

    Args from main superclass argparse.Action

    Attributes:
        action_values (list[str]): List of strings to accept as values used to
            check presence in with results of func attribute.

    """

    action_values = None

    def __init__(
        self,
        option_strings,
        dest,
        action_values=None,
        nargs=None,
        help=None,
        metavar=None,
    ):
        for attr in ["func", "err_msg_singular", "err_msg_plural"]:
            # Use getattr. hasattr returns True as they're initialized to None.
            if not getattr(self, attr):
                raise ValueError(
                    "Please supply required attribute: {}".format(attr)
                )

        if not action_values:
            # Raise ValueError if action_values not specified
            raise ValueError("Please supply required attribute: action_values")
        else:
            # Raise ValueError if action_values is not of type list
            if not isinstance(action_values, list):
                raise ValueError(
                    "Required attribute action_values has to be of type list"
                )
            # Raise ValueError if action_values list is empty
            elif len(action_values) == 0:
                raise ValueError(
                    "Required attribute action_values cannot be empty list"
                )
            else:
                # Accept init's action_values
                self.action_values = action_values

        super(CheckPresentInValuesAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            help=help,
            metavar=metavar,
        )

    def __call__(self, parser, namespace, values, option_string=None):
        # When values are a list of strings
        if isinstance(values, list):
            if not all(
                [
                    self.run_user_func(value) in self.action_values
                    for value in values
                ]
            ):
                raise argparse.ArgumentError(self, self.err_msg_plural)

        # When values is one string
        else:
            value = values
            if not self.run_user_func(value) in self.action_values:
                raise argparse.ArgumentError(self, self.err_msg_singular)

        setattr(namespace, self.dest, values)


class MapAction(BaseAction):
    """Maps func on values. Args from main suoperclass argparse.Action."""

    def __init__(
        self, option_strings, dest, nargs=None, help=None, metavar=None
    ):
        for attr in ["func"]:
            # Use getattr. hasattr returns True as they're initialized to None.
            if not getattr(self, attr):
                raise ValueError(
                    "Please supply required attribute: {}".format(attr)
                )

        super(MapAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            help=help,
            metavar=metavar,
        )

    def __call__(self, parser, namespace, values, option_string=None):
        # When values are a list of strings
        if isinstance(values, list):
            [self.run_user_func(value) for value in values]

        # When values is one string
        else:
            value = values
            self.run_user_func(value)

        setattr(namespace, self.dest, values)


class MapAndReplaceAction(BaseAction):
    """Maps func on values and replaces values with the result. Args from main
    superclass argparse.Action"""

    def __init__(
        self, option_strings, dest, nargs=None, help=None, metavar=None
    ):
        for attr in ["func"]:
            # Use getattr. hasattr returns True as they're initialized to None.
            if not getattr(self, attr):
                raise ValueError(
                    "Please supply required attribute: {}".format(attr)
                )

        super(MapAndReplaceAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            help=help,
            metavar=metavar,
        )

    def __call__(self, parser, namespace, values, option_string=None):
        # When values are a list of strings
        if isinstance(values, list):
            updated = [self.run_user_func(value) for value in values]
            values = updated

        # When values is one string
        else:
            value = values
            updated = self.run_user_func(value)
            values = updated

        setattr(namespace, self.dest, values)


class PipelineAction(ActionHeroAction):
    """Run ActionHero actions thrugh a pipeline.

    Actions that are run through a pipeline need to satify two constraints:
        1. Technical: Type need to match betwen piping and pipee actions.
        2. Logical: Piping some actions might not make logical sense.
            e.g. FilexxxActions to URLxxxActions

    Known Issues with Testing:
        Cases when the parser has pipeline action with multiple action_values
        due to the way argumenterrors are raised and exits.  Current workaround
        solution is to run those particular tests in a subprocess.  Note that
        this workaround WILL ONLY WORK when action_hero module is available
        e.g. with `pip install --editable .` which might not be the case on a
        different system. e.g. a CI system where action_hero module is not
        loaded.

    Attributes:
        children (list): Valid action_hero actions to pipeline through.
            Order of children is to be preserved.
        action_values (list[Action/(Action, [str])): Action values to this
            class contains a list of one of two valid options:
                1. [action_hero action]
                2. Tuple of (action_hero action, action_values<list>)

    """

    children = []
    action_values = None

    @staticmethod
    def _is_valid_action_hero_action(action):
        return issubclass(action, ActionHeroAction)

    def __init__(
        self,
        option_strings,
        dest,
        action_values=None,
        nargs=None,
        help=None,
        metavar=None,
    ):
        if not action_values:
            # Raise ValueError if action_values not specified
            raise ValueError("Please supply required attribute: action_values")
        else:
            # Raise ValueError if action_values is not of type list
            if not isinstance(action_values, list):
                raise ValueError(
                    "Required attribute action_values has to be of type list"
                )
            # Raise ValueError if action_values list is empty
            elif len(action_values) == 0:
                raise ValueError(
                    "Required attribute action_values cannot be empty list"
                )
            else:
                # Accept init's action_values
                self.action_values = action_values

        # Add actions as children
        for value in self.action_values:
            # Form 1 tuple of action class and it's action_values
            if isinstance(value, tuple):
                # 1. Get action class and action_values
                action = value[0]
                action_values = value[1]

                # 2. Verify the action is legit and raise ValueError if not
                if not self._is_valid_action_hero_action(action):
                    raise ValueError(
                        "Invalid action_hero action: {}".format(action)
                    )
                else:
                    # 3. Add action to children
                    self.children.append(
                        action(
                            option_strings=option_strings,
                            dest=dest,
                            nargs=nargs,
                            action_values=action_values,
                            help=help,
                            metavar=metavar,
                        )
                    )

            # Form 2 is an action class
            elif issubclass(value, argparse.Action):
                # 1. Get action class
                action = value
                # 2. Verify the action is legit and raise ValueError if not
                if not self._is_valid_action_hero_action(action):
                    raise ValueError(
                        "Invalid action_hero action: {}".format(action)
                    )
                # 3. Add action to children
                self.children.append(
                    action(
                        option_strings=option_strings,
                        dest=dest,
                        nargs=nargs,
                        help=help,
                        metavar=metavar,
                    )
                )

            # Raise ValueError if unexpected value
            else:
                raise ValueError("action_values contains invalid values")

        # REMAINDER PIPELINE INIT
        super(PipelineAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            nargs=nargs,
            help=help,
            metavar=metavar,
        )

    def __call__(self, parser, namespace, values, option_string=None):
        """Call each child action within PipelineAction's __call__

        All actions are called inside PipelineAction's namespace, thus end up
        using similar attributes such as namespace, parser etc. and as they
        each get called one after the other, they replace dest with their
        result. Thus they end up piping through results via `dest` in this
        namespace.

        No need to call setattr(namespace, self.dest, values) here, because
        we'll be leaving the result of the last action in the pipeline as it
        is and PipeineAction doesnt make any namespace changes

        """
        for child in self.children:
            child(
                parser=parser,
                namespace=namespace,
                values=values,
                option_string=option_string,
            )


class ExitCapturedArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        """error(message: string)

        Important note from this superclass's error(s, m) docstring:
            If you override this in a subclass, it should not return -- it
            should either exit or raise an exception.

        Reason for overriding superclass error:
            This function is overridden in order to stop the default behavior
            of ArgumentParser to exit when receiving an ArgumentError as that
            convolutes testing.
            So instead of exiting, just constructing the message and passing it
            on as a ValueError that can be captured in testing.

        Args:
            message(str): Use to explain reason for error.

        Raises:
            ValueError: Raises valueerror with formatted message.
        """
        error_message = "{}s: error: {}s\n".format(self.prog, message)
        raise ValueError(error_message)


class ActionHeroTestCase(unittest.TestCase):
    """unitests.TestCase subclass that encloses a ExitCapturedArgumentParser

    Reason for a special TestCase:
        1. Enclose parser within setup
        2. The enclosed parser should capture exits and raise ValueError

    """

    def setUp(self):
        """Enclose ExitCapturedArgumentParser as parser"""
        self.parser = ExitCapturedArgumentParser()



import argparse
import os
import tempfile
import unittest


from action_hero.utils import (
    ActionHeroAction,
    ActionHeroTestCase,
    ExitCapturedArgumentParser,
    run_only_when_modules_loaded,
)
from action_hero.utils import (
    BaseAction,
    CheckAction,
    CheckPresentInValuesAction,
    MapAction,
    MapAndReplaceAction,
    PipelineAction,
)
from action_hero import (
    FileDoesNotExistAction,
    FileExistsAction,
    FileIsEmptyAction,
    FileIsReadableAction,
    FileIsWritableAction,
    FilenameHasExtension,
)


@unittest.skip("TODO")
class TestRunOnlyWhenWhenInternetIsUp(ActionHeroTestCase):
    pass


class TestRunOnlyWhenModuleLoaded(unittest.TestCase):
    def test_on_available_module_unittest(self):
        @run_only_when_modules_loaded(modules=["sys"])
        def raise_value_error():
            raise ValueError()

        # Assert that func raise_value_error runs
        with self.assertRaises(ValueError):
            raise_value_error()

    def test_on_unavailable_module_unittest(self):
        @run_only_when_modules_loaded(modules=["unavailable_module_xyz"])
        def raise_value_error():
            raise ValueError()

        # func raise_value_error does not run
        raise_value_error()


class TestActionHeroTestCase(ActionHeroTestCase):
    def test_argparse_argument_parser_is_present(self):
        self.assertTrue(hasattr(self, "parser"))

    def test_argparse_argument_parser_is_instance_of_argument_parser(self):
        self.assertIsInstance(self.parser, argparse.ArgumentParser)

    def test_argparse_argument_parser_is_instance_of_custom_parser(self):
        self.assertIsInstance(self.parser, ExitCapturedArgumentParser)

    def test_is_subclassed_of_unittest_testcase(self):
        self.assertTrue(issubclass(ActionHeroTestCase, unittest.TestCase))


class TestBaseAction(ActionHeroTestCase):
    def test_if_is_subclass_of_argparse_action(self):
        self.assertTrue(issubclass(BaseAction, argparse.Action))


class TestCheckAction(ActionHeroTestCase):
    def test_if_is_subclass_of_argparse_action(self):
        self.assertTrue(issubclass(CheckAction, argparse.Action))

    def test_if_checks_for_required_func(self):
        class CheckActionWithotFunc(CheckAction):
            err_msg_singular = "S"
            err_msg_plural = "P"

        with self.assertRaises(ValueError):
            CheckActionWithotFunc(option_strings=[], dest="")

    def test_if_checks_for_required_err_msg_singular(self):
        class CheckActionWithotErrMsgSingular(CheckAction):
            func = print
            err_msg_plural = "P"

        with self.assertRaises(ValueError):
            CheckActionWithotErrMsgSingular(option_strings=[], dest="")

    def test_if_checks_for_required_err_msg_plural(self):
        class CheckActionWithotErrMsgSingular(CheckAction):
            func = print
            err_msg_singular = "S"

        with self.assertRaises(ValueError):
            CheckActionWithotErrMsgSingular(option_strings=[], dest="")


class TestMapAction(ActionHeroTestCase):
    def test_if_is_subclass_of_argparse_action(self):
        self.assertTrue(issubclass(MapAction, argparse.Action))

    def test_if_checks_for_required_func(self):
        class MapActionWithotFunc(MapAction):
            pass

        with self.assertRaises(ValueError):
            MapActionWithotFunc(option_strings=[], dest="")


class TestMapAndReplaceAction(ActionHeroTestCase):
    def test_if_is_subclass_of_argparse_action(self):
        self.assertTrue(issubclass(MapAndReplaceAction, argparse.Action))

    def test_if_checks_for_required_func(self):
        class MapAndReplaceWithotFunc(MapAndReplaceAction):
            pass

        with self.assertRaises(ValueError):
            MapAndReplaceWithotFunc(option_strings=[], dest="")


class TestCheckPresentInValuesAction(ActionHeroTestCase):
    def test_if_is_subclass_of_argparse_action(self):
        self.assertTrue(
            issubclass(CheckPresentInValuesAction, argparse.Action)
        )

    def test_if_checks_for_required_func_and_err_msgs(self):
        class CheckPresentInValuesActionSubClassWithoutFuncAndErrMsgs(
            CheckPresentInValuesAction
        ):
            pass

        with self.assertRaises(ValueError):
            CheckPresentInValuesActionSubClassWithoutFuncAndErrMsgs(
                option_strings=[], dest="", action_values=["UV"]
            )

    def test_if_checks_for_required_value(self):
        class CheckPresentInValuesActionSubClassWithoutValue(
            CheckPresentInValuesAction
        ):
            func = print
            err_msg_singular = "S"
            err_msg_plural = "P"

        with self.assertRaises(ValueError):
            CheckPresentInValuesActionSubClassWithoutValue(
                option_strings=[], dest=""
            )


class TestActionHeroAction(ActionHeroTestCase):
    def test_if_is_subclass_of_argparse_action(self):
        self.assertTrue(issubclass(ActionHeroAction, argparse.Action))


class TestPipelineAction(ActionHeroTestCase):
    """Known issue with testing PipelineActions using actions w/ action_values
    tends to cause errors where some test cases do not have any content for
    values"""
    def test_if_is_subclass_of_actionheroaction(self):
        self.assertTrue(issubclass(PipelineAction, ActionHeroAction))


    def test_on_action(self):
        self.parser.add_argument(
            "--file", action=PipelineAction, action_values=[FileExistsAction]
        )
        file1 = tempfile.mkstemp()[1]
        self.parser.parse_args(["--file", file1])
        os.remove(file1)

        # file1 now doesn't exist
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--file", file1])

    def test_on_multiple_actions(self):
        self.parser.add_argument(
            "--file",
            action=PipelineAction,
            action_values=[
                FileExistsAction,
                FileIsEmptyAction,
                FileIsWritableAction,
            ],
        )
        file1 = tempfile.mkstemp()[1]
        self.parser.parse_args(["--file", file1])
        # Can now confidently know that
        # 1. file1 exists
        # 2. file1 is empty
        # 3. file1 is writable
        with open(file1, "w") as f:
            f.write("write with no valuerrors")
        os.remove(file1)

    def test_on_multiple_actions_with_one_failure(self):
        self.parser.add_argument(
            "--file",
            nargs="+",
            action=PipelineAction,
            action_values=[
                FileExistsAction,
                FileIsEmptyAction,
                FileIsWritableAction,
            ],
        )
        file1 = tempfile.mkstemp()[1]
        self.parser.parse_args(["--file", file1])
        # Can now confidently know that
        # 1. file1 exists
        # 2. file1 is empty
        # 3. file1 is writable
        with open(file1, "w") as f:
            f.write("write with no valuerrors")

        # file1 no longer empty. Assert ValueError raised
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--file", file1])

        os.remove(file1)
        # file1 no longer exists. assert ValueError raised
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--file", file1])


class SoloTestCase(ActionHeroTestCase):
    def test_on_nonaction_hero_action_in_action_values(self):
        class UnrecognizedAction(argparse.Action):
            def __call__(self, parser, namespace, values, option_string=None):
                pass

        with self.assertRaises(ValueError):
            self.parser.add_argument(
                "--word",
                action=PipelineAction,
                action_values=[
                    UnrecognizedAction,
                ],
            )

    @unittest.skip("run only when action_hero module is available")
    def test_on_action_that_accepts_action_values(self):
        # 1. Code to run argumentparser and parse args
        script_contents = """
import argparse
from action_hero.utils import PipelineAction
from action_hero import FilenameHasExtension, FileDoesNotExistAction

# p = ExitCapturedArgumentParser()
p = argparse.ArgumentParser()
p.add_argument(
    "--readme",
    action=PipelineAction,
    nargs="+",
    action_values=[
        (FilenameHasExtension, ["md", "markdown"]),
        FileDoesNotExistAction,
    ],
)
args = p.parse_args(["--readme", "DAILY.md", "another.markdown"])
print(args.readme)
"""
        # 2. Create a temporary file
        with tempfile.NamedTemporaryFile(suffix=".py") as script_file:
            # 3. Write into temporary sciptfile
            with open(script_file.name, "w") as f:
                f.write(script_contents)

            # 4. Run temp_script_file as a subprocess
            from subprocess import PIPE, run
            import sys

            script_result = run(
                [sys.executable, script_file.name],
                stdin=PIPE,
                stdout=PIPE,
                stderr=PIPE,
            )
            # 5. Assert subprocess stderr is empty string
            self.assertFalse(script_result.stderr)



from action_hero.utils import CheckAction, CheckPresentInValuesAction

from action_hero.net_utils import (
    status_code_from_response_to_request_url,
    is_valid_ip_address,
    is_valid_ipv4_address,
    is_valid_ipv6_address,
    is_reachable_url,
)


__all__ = [
    "IPIsValidIPAddressAction",
    "IPIsValidIPAddressAction",
    "IPIsValidIPv4AddressAction",
    "IPIsValidIPv6AddressAction",
    "URLIsNotReachableAction",
    "URLIsReachableAction",
    "URLWithHTTPResponseStatusCodeAction",
]


class IPIsValidIPv4AddressAction(CheckAction):
    """Check if ip address is valid ipv4 address"""

    func = is_valid_ipv4_address
    err_msg_singular = "ip address is invalid ipv4 address"
    err_msg_plural = "Atleast one ip address is an invalid ipv4 address"


class IPIsValidIPv6AddressAction(CheckAction):
    """Check if ip address is valid ipv6 address"""

    func = is_valid_ipv6_address
    err_msg_singular = "ip address is invalid ipv6 address"
    err_msg_plural = "Atleast one ip address is an invalid ipv6 address"


class IPIsValidIPAddressAction(CheckAction):
    """Check if ip is valid ipv4 or ipv6 address"""

    func = is_valid_ip_address
    err_msg_singular = "ip address is invalid ip address"
    err_msg_plural = "Atleast one ip address is an invalid ipv(4/6) address"


class URLIsReachableAction(CheckAction):
    """Check if URL is reachable"""

    func = is_reachable_url
    err_msg_singular = "URL is not reachable"
    err_msg_plural = "Atleast one URL is not reachable"


class URLIsNotReachableAction(CheckAction):
    """Check if URL is not reachable"""

    def func(value):
        return not is_reachable_url(value)

    err_msg_singular = "URL is not reachable"
    err_msg_plural = "Atleast one URL is not reachable"


class URLWithHTTPResponseStatusCodeAction(CheckPresentInValuesAction):
    """Check if supplied URL responds with status code in action_values"""

    func = status_code_from_response_to_request_url
    err_msg_singular = "Response from URL does not have expected status code."
    err_msg_plural = (
        "Response from at least one URL does not have expected status code."
    )



from action_hero.email import EmailIsValidAction
from action_hero.net import (
    IPIsValidIPAddressAction,
    IPIsValidIPv4AddressAction,
    IPIsValidIPv6AddressAction,
    URLIsNotReachableAction,
    URLIsReachableAction,
    URLWithHTTPResponseStatusCodeAction,
)
from action_hero.path import (
    DirectoryDoesNotExistAction,
    DirectoryExistsAction,
    DirectoryIsExecutableAction,
    DirectoryIsNotExecutableAction,
    DirectoryIsNotReadableAction,
    DirectoryIsNotWritableAction,
    DirectoryIsReadableAction,
    DirectoryIsValidAction,
    DirectoryIsWritableAction,
    EnsureDirectoryAction,
    EnsureFileAction,
    FileDoesNotExistAction,
    FileExistsAction,
    FilenameHasExtension,
    FileIsEmptyAction,
    FileIsExecutableAction,
    FileIsNotEmptyAction,
    FileIsNotExecutableAction,
    FileIsNotReadableAction,
    FileIsNotWritableAction,
    FileIsReadableAction,
    FileIsWritableAction,
    PathDoesNotExistsAction,
    PathExistsAction,
    PathIsExecutableAction,
    PathIsNotExecutableAction,
    PathIsNotReadableAction,
    PathIsNotWritableAction,
    PathIsReadableAction,
    PathIsValidAction,
    PathIsWritableAction,
    ResolvePathAction,
)
from action_hero.path_utils import (
    add_execute_permission,
    is_empty_file,
    is_executable_directory,
    is_executable_file,
    is_existing_directory,
    is_existing_file,
    is_existing_path,
    is_readable_directory,
    is_readable_file,
    is_readable_path,
    is_valid_directory,
    is_valid_file,
    is_valid_path,
    is_writable_file,
    remove_execute_permission,
    remove_read_permission,
    remove_write_permission,
    resolve_path,
)
from action_hero.types import (
    IsConvertibleToFloatAction,
    IsConvertibleToIntAction,
    IsConvertibleToUUIDAction,
    IsFalsyAction,
    IsTruthyAction,
)

__all__ = ["EmailIsValidAction"]
__all__.extend(
    [
        "IPIsValidIPAddressAction",
        "IPIsValidIPAddressAction",
        "IPIsValidIPv4AddressAction",
        "IPIsValidIPv6AddressAction",
        "URLIsNotReachableAction",
        "URLIsReachableAction",
        "URLWithHTTPResponseStatusCodeAction",
    ]
)
__all__.extend(
    [
        "DirectoryDoesNotExistAction",
        "DirectoryExistsAction",
        "DirectoryIsExecutableAction",
        "DirectoryIsNotExecutableAction",
        "DirectoryIsNotReadableAction",
        "DirectoryIsNotWritableAction",
        "DirectoryIsReadableAction",
        "DirectoryIsValidAction",
        "DirectoryIsWritableAction",
        "EnsureDirectoryAction",
        "EnsureFileAction",
        "FileDoesNotExistAction",
        "FileExistsAction",
        "FilenameHasExtension",
        "FileIsEmptyAction",
        "FileIsExecutableAction",
        "FileIsNotEmptyAction",
        "FileIsNotExecutableAction",
        "FileIsNotReadableAction",
        "FileIsNotWritableAction",
        "FileIsReadableAction",
        "FileIsValidAction",
        "FileIsWritableAction",
        "PathDoesNotExistsAction",
        "PathExistsAction",
        "PathIsExecutableAction",
        "PathIsNotExecutableAction",
        "PathIsNotReadableAction",
        "PathIsNotWritableAction",
        "PathIsReadableAction",
        "PathIsValidAction",
        "PathIsWritableAction",
        "ResolvePathAction",
    ]
)
__all__.extend(
    [
        "IsConvertibleToFloatAction",
        "IsConvertibleToIntAction",
        "IsConvertibleToUUIDAction",
        "IsFalsyAction",
        "IsTruthyAction",
    ]
)



from action_hero.utils import ActionHeroTestCase
from action_hero.types import (
    IsConvertibleToFloatAction,
    IsConvertibleToIntAction,
    IsConvertibleToUUIDAction,
    IsFalsyAction,
    IsTruthyAction,
)


class TestIsConvertibleToIntAction(ActionHeroTestCase):
    def test_on_valid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToIntAction)
        value = "5"
        self.parser.parse_args(["--value", value])

    def test_on_valid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToIntAction
        )
        values = ["100", "45", "99", "0"]
        self.parser.parse_args(["--value", *values])

    def test_on_invalid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToIntAction)
        value = "icecream"
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", value])

    def test_on_invalid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToIntAction
        )
        values = ["puppy", "hanoi", "9.25", "X"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])

    def test_on_mixed_valid_and_invalid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToIntAction
        )
        values = ["4", "puppy", "hanoi", "9.25", "X", "12"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])


class TestIsConvertibleToFloatAction(ActionHeroTestCase):
    def test_on_valid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToFloatAction)
        value = "5.3"
        self.parser.parse_args(["--value", value])

    def test_on_valid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToFloatAction
        )
        values = ["100", "-4.3", "99", "0"]
        self.parser.parse_args(["--value", *values])

    def test_on_invalid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToFloatAction)
        value = "bunny"
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", value])

    def test_on_invalid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToFloatAction
        )
        values = ["puppy", "hanoi", "9.25", "X"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])

    def test_on_mixed_valid_and_invalid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToFloatAction
        )
        values = ["4", "puppy", "hanoi", "9.25", "X", "12"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])


class TestIsTruthyAction(ActionHeroTestCase):
    def test_on_truthy_value(self):
        self.parser.add_argument("--value", action=IsTruthyAction)
        value = "true"
        self.parser.parse_args(["--value", value])

    def test_on_truthy_values_list(self):
        self.parser.add_argument("--value", nargs="+", action=IsTruthyAction)
        values = ["true", "hundary"]
        self.parser.parse_args(["--value", *values])

    def test_on_falsy_value(self):
        self.parser.add_argument("--value", action=IsTruthyAction)
        value = ""
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", value])

    def test_on_falsy_values_list(self):
        self.parser.add_argument("--value", nargs="+", action=IsTruthyAction)
        values = ["", "0"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])

    def test_on_mixed_truthy_and_falsy_values_list(self):
        self.parser.add_argument("--value", nargs="+", action=IsTruthyAction)
        values = ["", "0", "monday", "15"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])


class TestIsFalsyAction(ActionHeroTestCase):
    def test_on_truthy_value(self):
        self.parser.add_argument("--value", action=IsFalsyAction)
        value = "true"
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", value])

    def test_on_truthy_values_list(self):
        self.parser.add_argument("--value", nargs="+", action=IsFalsyAction)
        values = ["true", "hundary"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])

    def test_on_falsy_value(self):
        self.parser.add_argument("--value", action=IsFalsyAction)
        value = ""
        self.parser.parse_args(["--value", value])

    def test_on_falsy_values_list(self):
        self.parser.add_argument("--value", nargs="+", action=IsFalsyAction)
        values = ["", "0"]
        self.parser.parse_args(["--value", *values])

    def test_on_mixed_truthy_and_falsy_values_list(self):
        self.parser.add_argument("--value", nargs="+", action=IsFalsyAction)
        values = ["", "0", "monday", "15"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])


class TestIsConvertibleToUUIDAction(ActionHeroTestCase):
    def test_on_valid_uuid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToUUIDAction)
        value = "cc3c3eb9-48a1-4307-8e92-700d9a25fffe"
        self.parser.parse_args(["--value", value])

    def test_on_valid_uppercase_uuid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToUUIDAction)
        value = "CC3C3EB9-48A1-4307-8E92-700D9A25FFFE"
        self.parser.parse_args(["--value", value])

    def test_on_valid_mixedcase_uuid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToUUIDAction)
        value = "cC3C3Eb9-48a1-4307-8e92-700d9a25fFFE"
        self.parser.parse_args(["--value", value])

    def test_on_valid_uuid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToUUIDAction
        )
        values = [
            "cc3c3eb9-48a1-4307-8e92-700d9a25fffe",
            "dfb55d07-340e-47e0-a3ba-0dc011369cef",
            "eca1f77c-e9c0-44c7-beeb-c8f7e0c59eb8",
        ]
        self.parser.parse_args(["--value", *values])

    def test_on_invalid_uuid_value(self):
        self.parser.add_argument("--value", action=IsConvertibleToUUIDAction)
        value = ("spaghetti-monster-juice",)
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", value])

    def test_on_invalid_uuid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToUUIDAction
        )
        values = ["spaghetti-monster-juice", "", "--4-beeb-c8f7e0c59eb8"]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])

    def test_on_mixed_valid_and_invalid_values_list(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToUUIDAction
        )
        values = [
            "spaghetti-monster-juice",
            "cc3c3eb9-48a1-4307-8e92-700d9a25fffe",
            "",
            "dfb55d07-340e-47e0-a3ba-0dc011369cef",
            "--4-beeb-c8f7e0c59eb8",
        ]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])

    def test_on_supported_uuid_forms(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToUUIDAction
        )
        values = [
            "urn:uuid:12345678-1234-5678-1234-567812345678",
            "ca761232ed4211cebacd00aa0057b223",
            "CA761232-ED42-11CE-BACD-00AA0057B223",
            "{CA761232-ED42-11CE-BACD-00AA0057B223}",
        ]
        self.parser.parse_args(["--value", *values])

    def test_on_unsupported_uuid_forms(self):
        self.parser.add_argument(
            "--value", nargs="+", action=IsConvertibleToUUIDAction
        )
        values = [
            "(CA761232-ED42-11CE-BACD-00AA0057B223)",
            "{}{}".format(
                "{0xCA761232, 0xED42, 0x11CE, {0xBA, 0xCD, ",
                "0x00, 0xAA, 0x00, 0x57, 0xB2, 0x23}}",
            ),
        ]
        with self.assertRaises(ValueError):
            self.parser.parse_args(["--value", *values])



import unittest

import requests

from action_hero.utils import run_only_when_when_internet_is_up
from action_hero.net_utils import (
    is_valid_ip_address,
    is_valid_ipv4_address,
    is_valid_ipv6_address,
    is_reachable_url,
    status_code_from_response_to_request_url,
)


class TestIsValidIPv4Address(unittest.TestCase):
    def test_on_valid_ipv6_address_string(self):
        self.assertFalse(is_valid_ipv4_address("2001:db8::1000"))

    def test_on_valid_ipv4_address_string(self):
        self.assertTrue(is_valid_ipv4_address("192.168.0.1"))

    def test_on_invalid_ipv4_address_string(self):
        self.assertFalse(is_valid_ipv4_address("500.168.0.1"))

    def test_on_valid_ipv4_address_numbers(self):
        self.assertTrue(is_valid_ipv4_address(3232235521))

    def test_on_invalid_ipv4_address_numbers(self):
        self.assertFalse(is_valid_ipv4_address(8000000000))

    def test_on_valid_ipv4_address_bytes(self):
        self.assertTrue(is_valid_ipv4_address(b"\xC0\xA8\x00\x01"))

    def test_on_invalid_ipv4_address_bytes(self):
        self.assertFalse(is_valid_ipv4_address(b"\xC0\xA8\x00"))


class TestIsValidIPv6Address(unittest.TestCase):
    def test_on_valid_ipv4_address_string(self):
        self.assertFalse(is_valid_ipv6_address("192.168.0.1"))

    def test_on_valid_ipv6_address_string(self):
        self.assertTrue(is_valid_ipv6_address("2001:db8::1000"))

    def test_on_invalid_ipv6_address_string(self):
        self.assertFalse(is_valid_ipv6_address("192.168.0.1"))

    def test_on_compressed_valid_ipv6_address_string(self):
        self.assertTrue(is_valid_ipv6_address("::1:1:1"))


class TestIsValidIPAddress(unittest.TestCase):
    def test_on_valid_ipv4_address_string(self):
        self.assertTrue(is_valid_ip_address("192.168.0.1"))

    def test_on_valid_ipv6_address_string(self):
        self.assertTrue(is_valid_ip_address("2001:db8::1000"))

    def test_on_invalid_ipv6_address_string(self):
        self.assertFalse(is_valid_ip_address("192.168.0.x"))

    def test_on_invalid_ipv4_address_bytes(self):
        self.assertFalse(is_valid_ip_address(b"\xC0\xA8\x00"))


class TestIsURLReachable(unittest.TestCase):
    @run_only_when_when_internet_is_up
    def test_on_reachable_url(self):
        url1 = "http://www.google.com"
        self.assertTrue(is_reachable_url(url1))

    def test_on_unreachable_url(self):
        url1 = "madeupurl.example.xyz"
        self.assertFalse(is_reachable_url(url1))


class TestStatusCodeFromResponseToRequestURL(unittest.TestCase):
    @run_only_when_when_internet_is_up
    def test_on_reachable_url(self):
        try:
            url1 = "http://www.google.com"
            response = requests.get(url1)  # Raises RequestException on fail
            self.assertEqual(
                status_code_from_response_to_request_url(url1),  # None on fail
                response.status_code,
            )

        # Do nothing on response error
        except requests.exceptions.RequestException:
            pass

    def test_on_unreahable_url(self):
        try:
            url1 = "madeupurl.example.xyz"
            response = requests.get(url1)  # Raises RequestException on fail
            self.assertEqual(
                status_code_from_response_to_request_url(url1),  # None on fail
                response.status_code,
            )

        # Do nothing on response error
        except requests.exceptions.RequestException:
            pass

    def test_on_malformed_url_returns_none_on_request_failure(self):
        url1 = "AAA"
        self.assertIsNone(status_code_from_response_to_request_url(url1))



from action_hero.email_utils import is_valid_email
from action_hero.utils import CheckAction


__all__ = ["EmailIsValidAction"]


class EmailIsValidAction(CheckAction):
    """Check if email address is valid"""

    func = is_valid_email
    err_msg_singular = "Email address is invalid"
    err_msg_plural = "Atleast one supplied email address is invalid"



import uuid


__all__ = [
    "is_convertible_to_UUID",
    "is_convertible_to_float",
    "is_convertible_to_int",
    "is_truthy",
]


def is_convertible_to_int(value):
    """Returns True if value is convertible to int"""
    try:
        int(value)
    except ValueError:
        return False
    else:
        return True


def is_convertible_to_float(value):
    """Returns True if value is convertible to float"""
    try:
        float(value)
    except ValueError:
        return False
    else:
        return True


def is_truthy(value):
    """Returns True if value is truthy

    Only attempting truthy when is string of zeroes 0/0.0

    Args:
        value (str): Value to check truthy for.

    Returns:
        bool: True if deemed truthy else False
    """
    try:
        # 1. Check if can be converted to a float
        float(value)

    except ValueError:
        # 2. Return truthy of value if cannot be converted to float
        return bool(value)

    else:
        # 3. Return truthy of value if can be converted to a float
        return bool(float(value))


def is_convertible_to_UUID(value, version=4):
    """Returns True if value is convertible to UUIDs

    UUID v.4 General Format:
      - xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx
      - where x is a hex digit and y is one of (8, 9, A, B)

    Using python uuid.UUID and as so,
    Supporting these forms:
      ca761232ed4211cebacd00aa0057b223
      CA761232-ED42-11CE-BACD-00AA0057B223
      {CA761232-ED42-11CE-BACD-00AA0057B223}
    Not supporting these forms
      (CA761232-ED42-11CE-BACD-00AA0057B223)
      {0xCA761232, 0xED42, 0x11CE, {0xBA, 0xCD, 0x00, 0xAA, *LINER BREAK*
      0x00, 0x57, 0xB2, 0x23}}

    Args:
        value (str): Value to attempt to convert to an UUID
        version (int): UUID version. Default is UUIDv4

    Returns:
        bool: True if deemed valid UUID

    """
    try:
        # 1. Check if can be converted to a UUID
        uuid.UUID(value, version=version)

    except (ValueError, AttributeError):
        # 2. Return false if cannot be converted to UUID
        return False

    else:
        # 3. Return true if can be converted to UUID
        return True



import re


__all__ = [
    "is_valid_email",
]


def is_valid_email(email):
    """Return True if email is valid

    There is no perfect regex match for email.
    Using the email regex pattern from http://emailregex.com

    Args:
        email (str): The email address to check validity for
    """
    pattern = re.compile(r"(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)")
    return True if pattern.match(email) else False



import unittest

from action_hero.getters import get_about


class TestAbout(unittest.TestCase):
    def setUp(self):
        self.about = get_about()

    def test_about_is_dict(self):
        self.assertIsInstance(self.about, dict)

    def test_about_is_not_empty(self):
        self.assertGreater(len(self.about), 0)


class TestAboutProject(unittest.TestCase):
    def setUp(self):
        self.about = get_about()
        self.project = self.about["PROJECT"]

    def test_has_section_project(self):
        self.assertIn("PROJECT", self.about)

    def test_section_project_has_program_name(self):
        self.assertIn("program_name", self.project)

    def test_section_project_has_description(self):
        self.assertIn("description", self.project)

    def test_section_project_has_version(self):
        self.assertIn("version", self.project)

    def test_section_project_has_license(self):
        self.assertIn("license", self.project)

    def test_section_author_has_homepage(self):
        self.assertIn("url", self.project)


class TestAboutAuthor(unittest.TestCase):
    def setUp(self):
        self.about = get_about()
        self.author = self.about["AUTHOR"]

    def test_authohas_section_author(self):
        self.assertIn("AUTHOR", self.about)

    def test_section_author_has_name(self):
        self.assertIn("name", self.author)

    def test_section_author_has_email(self):
        self.assertIn("email", self.author)



from configparser import ConfigParser, ExtendedInterpolation
import pathlib

try:
    # importlib is available only from Python 3.7+
    from importlib import resources
except ImportError:
    # importlib_resources package is available for Python 3.4-3.6
    import importlib_resources as resources

import meta


def get_about():
    """Return contents of about.ini as a dict"""
    return get_config_as_dict(meta, "about.ini")


def get_readme_contents():
    """Return contents of file marked readme_filename in about.ini"""
    about = get_about()
    readme_filename = about["PROJECT"]["readme_filename"]
    with open(readme_filename, "r", encoding="utf-8") as f:
        readme_contents = f.read()

    return readme_contents


def get_readme_content_type():
    """Return content-type of readme

    Readme is deduced from file marked readme_filename in about.ini

    Returns:
        str: content-type of readme file
    """
    about = get_about()
    suffix = pathlib.Path(about["PROJECT"]["readme_filename"]).suffix
    ext_and_content_types = {"md": "text/markdown", "rst": "text/x-rst"}
    return ext_and_content_types.get(suffix, "text/plain")


def get_config_as_dict(resource_path, config_filename):
    """Returns contents of a config file

    The `resource_path` module needs to be importable and importlib needs to be
    imported importlib is only 3.7+, so use external library
    importlib_resources to fill in the gap for previous versions of python

    Args:
        resource_path (module): Module to fetch resource from.
        config_filename (str): ini file to fetch from resource_path

    Returns:
        (str): Contents of config file
    """
    with resources.path(resource_path, config_filename) as config_path:
        config = ConfigParser(
            delimiters=("="),
            comment_prefixes=("#"),
            inline_comment_prefixes=("#"),
            interpolation=ExtendedInterpolation(),
        )
        config.read(config_path)
        return dict(config)



from action_hero.utils import (
    CheckAction,
    CheckPresentInValuesAction,
    MapAction,
    MapAndReplaceAction,
)
from action_hero.path_utils import (
    create_directory,
    create_file,
    get_extension,
    is_empty_file,
    is_executable_directory,
    is_executable_file,
    is_executable_path,
    is_existing_directory,
    is_existing_file,
    is_existing_path,
    is_readable_directory,
    is_readable_file,
    is_readable_path,
    is_valid_directory,
    is_valid_file,
    is_valid_path,
    is_writable_directory,
    is_writable_file,
    is_writable_path,
    resolve_path,
)


__all__ = [
    "DirectoryDoesNotExistAction",
    "DirectoryExistsAction",
    "DirectoryIsExecutableAction",
    "DirectoryIsNotExecutableAction",
    "DirectoryIsNotReadableAction",
    "DirectoryIsNotWritableAction",
    "DirectoryIsReadableAction",
    "DirectoryIsValidAction",
    "DirectoryIsWritableAction",
    "EnsureDirectoryAction",
    "EnsureFileAction",
    "FileDoesNotExistAction",
    "FileExistsAction",
    "FileIsEmptyAction",
    "FileIsExecutableAction",
    "FileIsNotEmptyAction",
    "FileIsNotExecutableAction",
    "FileIsNotReadableAction",
    "FileIsNotWritableAction",
    "FileIsReadableAction",
    "FileIsValidAction",
    "FileIsWritableAction",
    "FilenameHasExtension",
    "PathDoesNotExistsAction",
    "PathExistsAction",
    "PathIsExecutableAction",
    "PathIsNotExecutableAction",
    "PathIsNotReadableAction",
    "PathIsNotWritableAction",
    "PathIsReadableAction",
    "PathIsValidAction",
    "PathIsWritableAction",
    "ResolvePathAction",
]


class ResolvePathAction(MapAndReplaceAction):
    """Resolves path to canonical path removing symbolic links if present"""

    func = resolve_path


class EnsureDirectoryAction(MapAction):
    """Ensure directory exists and create it if it doesnt"""

    @staticmethod
    def _ensure_directory(directory):
        # create_directory raises FileExistsError if dir exists so check first
        if not is_existing_directory(directory):
            create_directory(directory)

    func = _ensure_directory


class EnsureFileAction(MapAction):
    """Ensure file exists and create it if it doesnt"""

    @staticmethod
    def _ensure_file(filename):
        if not is_existing_file(filename):
            create_file(filename)

    func = _ensure_file


class PathIsValidAction(CheckAction):
    """Check if path is valid"""

    func = is_valid_path
    err_msg_singular = "Path is invalid"
    err_msg_plural = "Atleast one path is invalid"


class PathExistsAction(CheckAction):
    """Check if path exists"""

    func = is_existing_path
    err_msg_singular = "Path does not exist"
    err_msg_plural = "Atleast one path does not exist"


class PathDoesNotExistsAction(CheckAction):
    """Check if path does not exist"""

    def func(value):
        return not is_existing_path(value)

    err_msg_singular = "Path exists"
    err_msg_plural = "Atleast one path exists"


class PathIsWritableAction(CheckAction):
    """Check if path is writable"""

    func = is_writable_path
    err_msg_singular = "Path is not writable"
    err_msg_plural = "Atleast one path is not writable"


class PathIsNotWritableAction(CheckAction):
    """Check if path is not writable"""

    def func(value):
        return not is_writable_path(value)

    err_msg_singular = "Path is writable"
    err_msg_plural = "Atleast one path is writable"


class PathIsReadableAction(CheckAction):
    """Check if path is readable"""

    func = is_readable_path
    err_msg_singular = "Path is not readable"
    err_msg_plural = "Atleast on path is not readable"


class PathIsNotReadableAction(CheckAction):
    """Check if path is not writable"""

    def func(value):
        return not is_readable_path(value)

    err_msg_singular = "Path is readable"
    err_msg_plural = "Atleast on path is readable"


class PathIsExecutableAction(CheckAction):
    """Check if path is executable"""

    func = is_executable_path
    err_msg_singular = "Path is not executable"
    err_msg_plural = "Atleast one path is not executable"


class PathIsNotExecutableAction(CheckAction):
    """Check if path is not executable"""

    def func(value):
        return not is_executable_path(value)

    err_msg_singular = "Path is executable"
    err_msg_plural = "Atleast one path is executable"


class DirectoryExistsAction(CheckAction):
    """Check if directory exists"""

    func = is_existing_directory
    err_msg_singular = "Directory does not exist"
    err_msg_plural = "Atleast one directory does not exist"


class DirectoryDoesNotExistAction(CheckAction):
    """Check if directory does not exist"""

    def func(value):
        return not is_existing_directory(value)

    err_msg_singular = "Directory exists"
    err_msg_plural = "Atleast one directory exists"


class DirectoryIsWritableAction(CheckAction):
    """Check if directory is writable"""

    func = is_writable_directory
    err_msg_singular = "Directory is not writable"
    err_msg_plural = "Atleast one directory is not writable"


class DirectoryIsNotWritableAction(CheckAction):
    """Check if directory is not writable"""

    def func(value):
        return not is_writable_directory(value)

    err_msg_singular = "Directory is not writable"
    err_msg_plural = "Atleast one directory is not writable"


class DirectoryIsReadableAction(CheckAction):
    """Check if directory is readable"""

    func = is_readable_directory
    err_msg_singular = "Directory is not readable"
    err_msg_plural = "Atleast one directory is not readable"


class DirectoryIsNotReadableAction(CheckAction):
    """Check if directory is not readable"""

    def func(value):
        return not is_readable_directory(value)

    err_msg_singular = "Directory is readable"
    err_msg_plural = "Atleast one directory is readable"


class DirectoryIsExecutableAction(CheckAction):
    """Check if directory is executable"""

    func = is_executable_directory
    err_msg_singular = "Directory is not executable"
    err_msg_plural = "Atleast one directory is not executable"


class DirectoryIsNotExecutableAction(CheckAction):
    """Check if directory is not executable"""

    def func(value):
        return not is_executable_directory(value)

    err_msg_singular = "Directory is executable"
    err_msg_plural = "Atleast one directory is executable"


class DirectoryIsValidAction(CheckAction):
    """Check directory is valid"""

    func = is_valid_directory
    err_msg_singular = "Directory is not valid"
    err_msg_plural = "Atleast one directory is not valid"


class FileIsWritableAction(CheckAction):
    """Check if file is writable"""

    func = is_writable_file
    err_msg_singular = "File is not writable"
    err_msg_plural = "Atleast one file is not writable"


class FileIsNotWritableAction(CheckAction):
    """Check if file is not writable"""

    def func(value):
        return not is_writable_file(value)

    err_msg_singular = "File is writable"
    err_msg_plural = "Atleast one file is writable"


class FileIsReadableAction(CheckAction):
    """Check if file is readable"""

    func = is_readable_file
    err_msg_singular = "File is not readable"
    err_msg_plural = "Atleast one file is not readable"


class FileIsNotReadableAction(CheckAction):
    """Check if file is not readable"""

    def func(value):
        return not is_readable_file(value)

    err_msg_singular = "File is readable"
    err_msg_plural = "Atleast one file is readable"


class FileIsExecutableAction(CheckAction):
    """Check if file is executable"""

    func = is_executable_file
    err_msg_singular = "File is not executable"
    err_msg_plural = "Atleast one file is not executable"


class FileIsNotExecutableAction(CheckAction):
    """Check if file is not executable"""

    def func(value):
        return not is_executable_file(value)

    err_msg_singular = "File is executable"
    err_msg_plural = "Atleast one file is executable"


class FileIsValidAction(CheckAction):
    """Check file is valid"""

    func = is_valid_file
    err_msg_singular = "File is not valid"
    err_msg_plural = "Atleast one file is not valid"


class FileExistsAction(CheckAction):
    """Check if file exists"""

    func = is_existing_file
    err_msg_singular = "File does not exist"
    err_msg_plural = "Atleast one file does not exist"


class FileDoesNotExistAction(CheckAction):
    """Check if file exists"""

    def func(value):
        return not is_existing_file(value)

    err_msg_singular = "File exists"
    err_msg_plural = "Atleast one file exists"


class FileIsEmptyAction(CheckAction):
    """Check if file is empty"""

    func = is_empty_file
    err_msg_singular = "File is not empty"
    err_msg_plural = "Atleast one file is not empty"


class FileIsNotEmptyAction(CheckAction):
    """Check if file is not empty"""

    def func(value):
        return not is_empty_file(value)

    err_msg_singular = "File is empty"
    err_msg_plural = "Atleast one file is empty"


class FilenameHasExtension(CheckPresentInValuesAction):
    """Check if file has specified extension"""

    func = get_extension
    err_msg_singular = "File does not have expected extension"
    err_msg_plural = "Atleast one file does not have expected extension"



import argparse

from action_hero import FileIsReadableAction


if __name__ == "__main__":
    # Create parser
    parser = argparse.ArgumentParser()

    # Add user argument "file"
    parser.add_argument("--file", action=FileIsReadableAction)

    # Parse user arguments
    args = parser.parse_args()

    if args.file:
        # Count lines in file
        with open(args.file) as f:
            # print(f"{args.file} has {len(f.readlines())} lines")
            print("{} has {} lines".format(args.file, len(f.readlines())))
    else:
        # Print usage when no arguments were supplied
        parser.print_usage()
